////////////////////////////////////////////////////////////////////////////////
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
// http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software 
// distributed under the License is distributed on an "AS IS" BASIS, 
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and 
// limitations under the License
// 
// No warranty of merchantability or fitness of any kind. 
// Use this software at your own risk.
// 
////////////////////////////////////////////////////////////////////////////////
package com.balsamiq2flexjs
{
	import flash.desktop.NativeApplication;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.setTimeout;
	import flash.xml.XMLNode;
	import flash.xml.XMLNodeType;
	
	import mx.controls.Alert;
	
	public class BalsamiqToFlexJSConvert extends Sprite
	{
		public static const IS_MACOS:Boolean = !NativeApplication.supportsSystemTrayIcon;
		
		private var calledCount:int = 0;
		private var xmlData:XML;
		private var arrXMLData:Array;
		
		// Domain class properties, used by single object view and other files
		private var TableName:String;
		private var GrailsDomainClassPackageName:String;
		private var GrailsDomainClassName:String;
		private var GrailsSanitizerType:String;
		private var RESTUrl:String;
		
		
		// Domain class fields properties, used by FlexJS domain class and utils
		// NOTE the single object view doesn't use these, since it uses old custom code to generate
		private var domainClassFields: Array;
		//private var ColumnFields: Array;
		private var ColumnFieldsInConfiguration: String;
		
		// Used by single object view
		private var arrControls:Array;
		private var arrComponents:Array;
		
		
		private var strMXMLData:String;
		private var arrViewMXMLData:Array;
		
		private var strMXMLExportFilePath:String;
		private var XVarianceAllowed:int;
		private var YVarianceAllowed:int;
		private var Debugging:Boolean;
		private var blnValidXML:Boolean;
		
		private var isMultiFile:Boolean;
		
		private var strXMLPath : String;
		
		private var fileNameArrays:Array;
		// domain files, one domain can have more than 1 bmml for now.
		private var domains:Array = new Array();
		
		private var fileStartCount:int;
		private var fileEndCount:int;
		private var allPhysicalFilesExist:Boolean;
		private var FilesPath:String;
		private var projectName:String;
		
		private var templatesFolder:File = File.applicationDirectory.resolvePath("templates");
		
		public function isValidXML():Boolean
		{
			return blnValidXML;
		}
		
		public function setProjectName(_projectName:String):void
		{
			projectName=_projectName;
		}
		
		
		private function getPackageName(): String
		{
			return GrailsDomainClassPackageName;
		}
		
		private function getPackageAsFilePath(): String
		{
			if (getPackageName() != null) 
			{
				
				return getPackageName().split(".").join("/");
				
			} else {
				
				return null;
			}
		}
		
		private function getDomainName():String
		{
			return GrailsDomainClassName;
		}
		
		
		
		public function BalsamiqToFlexJSConvert(XMLPath:String, MXMLExportFilePath:String, singleFile:Boolean = true):void
		{
			try{
				var intIndex1:int;
				var intIndex2:int;
				var strFileName:String;
				var strFileNameFirstPart:String;
				var strFileNameToPush:String;
				fileNameArrays = new Array();
				domains = new Array();
				arrXMLData = new Array();
				var domainMainFiles: Array = new Array();
				
				if (singleFile)
				{
					domainMainFiles.push(new File(XMLPath));
				} else {
					
					if (new File(XMLPath).isDirectory) 
					{
						var files : Array = new File(XMLPath).getDirectoryListing();
						for each (var oneFile:File in files)
						{
							if (oneFile.url.indexOf("_of_") > -1 && oneFile.url.indexOf("1_of_") == -1)
							{
								continue;
							} else {
								domainMainFiles.push(oneFile);
							}
						}
						
					}
				}
				
				for each (var oneDomainModelFile: File in domainMainFiles)
				{
					var strOneDomainPath:String = oneDomainModelFile.url;
					
					//strXMLPath = "file://" + strXMLPath;
					strMXMLExportFilePath = MXMLExportFilePath;
					var file:File = new File(strMXMLExportFilePath);
					file.createDirectory();
					blnValidXML = true;
					
					intIndex1 = 0;
					intIndex2 = strOneDomainPath.lastIndexOf("/");
					
					FilesPath = strOneDomainPath.substring(intIndex1, intIndex2);
					
					strFileName=strOneDomainPath.substring(intIndex2+1);
					
					
					intIndex1 = strFileName.indexOf("_of_");
					intIndex2 = strFileName.lastIndexOf("_", intIndex1 - 1);
					strFileNameFirstPart = strFileName.substring(0, intIndex2);
					
					
					intIndex1 = strOneDomainPath.indexOf("_of_");
					//fileNameArrays;
					var fileNames = new Array();
					fileNameArrays.push(fileNames);
					if (intIndex1 < 0)
					{
						strFileNameToPush = FilesPath + "/" + strFileName;
						isMultiFile = false;
						fileNames.push(strFileNameToPush);
					}
					else
					{
						var strValue:String;
						var intKtr:int;
						isMultiFile = true;
						intIndex2 = strOneDomainPath.indexOf(".bmml");
						strValue = strOneDomainPath.substring(intIndex1+4, intIndex2);
						
						fileEndCount = Number(strValue);
						
						fileStartCount = 1;
						
						for (intKtr = fileStartCount; intKtr <= fileEndCount; intKtr++)
						{
							strFileNameToPush = FilesPath + "/" + strFileNameFirstPart + "_" + intKtr + "_of_" + fileEndCount + ".bmml";
							
							fileNames.push(strFileNameToPush);
						}
						
					}
				}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		private var currentDomainFile: int = 0;
		
		public function processNextDomain(mDebugging:Boolean): void
			
		{
			if (currentDomainFile< 0 || currentDomainFile>=fileNameArrays.length)
			{
				//TODO finish
				return;
			}
			try{
				for each (var fileNames: Array in fileNameArrays) {
					
					var intLen:int = fileNames.length;
					var intKtr:int;
					
					for (intKtr = 0; intKtr < intLen; intKtr++)
					{
						var myXML:XML = new XML();
						var XML_URL:String = fileNames[intKtr];
						
						
						Debugging = mDebugging;
						
						if (intKtr < intLen - 1)
						{
							var myXMLURL1:URLRequest = new URLRequest(XML_URL);
							var myLoader1:URLLoader = new URLLoader(myXMLURL1);
							
							myLoader1.addEventListener(Event.COMPLETE, xmlDownloadedMulti);
						}
						else if (intKtr == intLen - 1)
						{
							var myXMLURL2:URLRequest = new URLRequest(XML_URL);
							var myLoader2:URLLoader = new URLLoader(myXMLURL2);
							
							myLoader2.addEventListener(Event.COMPLETE, xmlDownloadedMultiComplete);
						}
						
					}
				}
				
			}
			catch (e:Error)
			{
				Alert.show("Could not open the XML file for processing:  \"" + strXMLPath + "\"");
				blnValidXML = false;
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			currentDomainFile++;
			
		}
		public function loadXMLMulti(mDebugging:Boolean):void
		{
			// produce project file and navigator views
			
			var isSingle = isWritingSingleDomainProject();
			
			// These two files should be produced only once for all input models
			// moonshineide project file (fixed file content)
			//trace("Export project file");
		
			if (isSingle)
			{
				//exportMainViewsOfSingleDomainProject(isSingle);
			} else {
				exportProjectFile(isSingle);
				exportMainViews(isSingle);
				// TODO out side the loop
			}
			
			
			
			currentDomainFile = 0;
			
			processNextDomain(mDebugging);
			/*
			try{
				for each (var fileNames: Array in fileNameArrays) {
					
					var intLen:int = fileNames.length;
					var intKtr:int;
					
					for (intKtr = 0; intKtr < intLen; intKtr++)
					{
						var myXML:XML = new XML();
						var XML_URL:String = fileNames[intKtr];
						
						
						Debugging = mDebugging;
						
						if (intKtr < intLen - 1)
						{
							var myXMLURL1:URLRequest = new URLRequest(XML_URL);
							var myLoader1:URLLoader = new URLLoader(myXMLURL1);
							
							myLoader1.addEventListener(Event.COMPLETE, xmlDownloadedMulti);
						}
						else if (intKtr == intLen - 1)
						{
							var myXMLURL2:URLRequest = new URLRequest(XML_URL);
							var myLoader2:URLLoader = new URLLoader(myXMLURL2);
							
							myLoader2.addEventListener(Event.COMPLETE, xmlDownloadedMultiComplete);
						}
						
					}
				}
				
			}
			catch (e:Error)
			{
				Alert.show("Could not open the XML file for processing:  \"" + strXMLPath + "\"");
				blnValidXML = false;
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			*/
		}
		
		private function onXMLDownloadIO(event:IOErrorEvent):void
		{
			trace(event.text);
		}
		
		private  var callBackF:Function;
		
		public function validateXML(callback:Function):void
		{
			try{
				var myXMLURL:URLRequest = new URLRequest(strXMLPath);
				var myLoader:URLLoader = new URLLoader(myXMLURL);
				
				callBackF = callback;
				
				myLoader.addEventListener(Event.COMPLETE, xmlValidation);
			}
			catch (e:Error)
			{
				Alert.show("Could not open the XML file for validation:  \"" + strXMLPath + "\"");
				blnValidXML = false;
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		private function xmlValidation(event:Event):void
		{
			try{
				xmlData = new XML(event.target.data);
				callBackF();
			}
			catch (e:Error)
			{
				Alert.show("The XML file \"" + strXMLPath + "\" could not be parsed.");
				blnValidXML = false;
				callBackF();
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		private function xmlDownloadedMulti(event:Event):void
		{
			try{
				xmlData = new XML(event.target.data);
				
				arrXMLData.push(xmlData);
			}
			catch (e:Error)
			{
				Alert.show("The XML file \"" + strXMLPath + "\" could not be parsed during processing.");
				blnValidXML = false;
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		// Codes affecting output files begin here
		
		// Codes affecting output project structure only
		private function collectMXMLData():void
		{
			try{
				
				var intLen:int = arrComponents.length;
				var intKtr:int;
				arrViewMXMLData = new Array();
				if(intLen>0)
					strMXMLData = "";
				
				trace("******inside collectMXMLData,intLen=" + intLen);
				
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					//trace("******" + arrComponents[intKtr].controlID + " is custom =" +  (arrComponents[intKtr].controlTypeID == "custom"));
					if (arrComponents[intKtr].controlTypeID == "custom")
					{
						if (arrComponents[intKtr].IsChildCustomTAB)
						{
							var intKtr2:int;
							var intTABCount:int;
							intTABCount = arrComponents[intKtr].ChildCustomTABsData.length;
							
							for (intKtr2 = 0; intKtr2 < intTABCount; intKtr2++)
							{
								
								var strFileName:String;
								
								strFileName = arrComponents[intKtr].ChildCustomTABsData[intKtr2].TABName;
								strFileName = strFileName.replace(" ", "").replace(" ", "").replace(" ", "").replace(" ", "").replace(" ", "").replace(" ", "").replace("-", "").replace("-", "").replace("\\", "").replace("/", "").replace("\\", "").replace("/", "");
								strFileName = strFileName + "Tab.mxml";
								trace("****** push sub tab file=" + strFileName + "controlID=" + arrComponents[intKtr].controlID  + " controlTypeID=" + arrComponents[intKtr].controlTypeID)
								arrViewMXMLData.push({fileName: strFileName, data: arrComponents[intKtr].ChildCustomTABsData[intKtr2].text});
								
							}
						}
						strMXMLData = strMXMLData + arrComponents[intKtr].text;
					}
				}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		private function getObjectViewFileName():String
		{
			
			return strMXMLExportFilePath + "/src/views/" + getDomainName() + "/" + getDomainName() +  "View.mxml";
		}
		
		
		private function getTabViewFileName(name : String): String
		{
			return strMXMLExportFilePath + "/src/views/" + getDomainName() + "/" + name;
		}
		
		
		// syntax strings
		/* Old version using FlexJS:HTML
		private function mainViewFileHeader():String
		{
		return "<?xml version=\"1.0\" encoding=\"utf-8\"?> \n<js:View  xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" xmlns:local=\"*\" xmlns:layout=\"layout.*\" xmlns:views=\"views.*\">"
		}
		
		private function mainViewFileFooter(): String
		{
		return "</js:View>"
		}
		
		private function tabGroupHeader( strIDTabGroup:String,arrayTabTitles:Array,maxTabHeightInCurrentGroup:int, width:int ) : String 
		{
		var strMXMLData: String = "";
		strMXMLData = strMXMLData + "<js:Container  id=\"CTabBar" + strIDTabGroup + "\" height=\"" + (maxTabHeightInCurrentGroup + 70) + "\">";
		strMXMLData = strMXMLData + "<js:beads><js:OneFlexibleChildVerticalLayout flexibleChild=\"tabbarview" + strIDTabGroup + "\"/></js:beads>";
		
		strMXMLData = strMXMLData + "<layout:TabBar dataProvider=\"{tabTitles" + (arrayTabTitles.length + 1) + "}\" width=\"" + width + "\" height=\"50\" id=\"tabbar"+ strIDTabGroup + "\" viewContainer=\"{tabbarview" + strIDTabGroup + "}\" />";
		strMXMLData = strMXMLData + "<layout:TabBarView id=\"tabbarview" + strIDTabGroup + "\" width=\"50%\" height=\"50%\" x= \"1\" y=\"30\">";
		return strMXMLData;
		}
		
		
		
		
		private function mxmlFileHead(_IsInitialView: Boolean): String
		{
		if(_IsInitialView)
		// No need to update to MDL? if yes, then it is not clear which component can be used
		return "<js:View xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" \n xmlns:layout=\"layout.*\" xmlns:views = \"views.*\">\n";
		else
		//return  "<js:Container xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" \n xmlns:layout=\"layout.*\" xmlns:views = \"views.*\">\n";
		return '<mdl:Grid ' +
		'xmlns:fx="http://ns.adobe.com/mxml/2009"\n' +
		'xmlns:js="library://ns.apache.org/flexjs/basic"\n' +
		'xmlns:mdl="library://ns.apache.org/flexjs/mdl"\n' +
		'xmlns="http://www.w3.org/1999/xhtml"' +
		'xmlns:layout="layout.*"' +
		'xmlns:views="views.*"' +
		'>'
		
		}
		
		private function mxmlFileFoot(_IsInitialView: Boolean): String
		{
		if(_IsInitialView)
		return "\n</js:View>";
		else
		//return "\n</js:Container>";
		return "\n</mdl:Grid>";
		}
		
		
		private function tabCode(strTABNameNoSpace : String) : String
		{
		return "<views:"+strTABNameNoSpace+"Tab id=\""+strTABNameNoSpace+"\"/>";
		}
		
		
		private function tabGroupFooter(): String
		{
		return 	 "</layout:TabBarView>\n" + "</js:Container>";
		}
		
		private function tabGroupFooter2(): String
		{
		// not sure if it is used at all
		return  "</layout:TabBarView>\n" + "</js:Container>";
		}
		
		private function getFXScript(arrayTabTitles:Array, arrayTabIds:Array,  strFieldNames:String, isCustomTAB:Boolean):String
		{
		var strReturn:String = "";
		
		strReturn = strReturn + "<fx:Script>"+"\n";
		strReturn = strReturn + "<![CDATA["+"\n";
		if (isCustomTAB) 
		{
		strReturn = strReturn + "		" + "import events.TabItemClickedEvent;" + "\n";
		strReturn = strReturn + "		" + "import org.apache.flex.events.Event;" + "\n";
		strReturn = strReturn + "		" + "import org.apache.flex.html.SimpleAlert;" + "\n";
		}
		strReturn +="		" + 'import org.apache.flex.events.CustomEvent;\n';
		strReturn +="		" + 'import org.apache.flex.events.Event;\n';
		
		strReturn +="		" +  "public var _controls:Array = new Array(" + strFieldNames + ");" + "\n";
		strReturn +="		" +  "[Bindable(\"__NoChangeEvent__\")]"+"\n";
		strReturn +="		" +  "public function get controls():Array"+"\n";
		strReturn +="		" +  "{"+"\n";
		strReturn +="		" +  "    return _controls;" + "\n";
		strReturn +="		" +  "}"+"\n";
		
		var indexTabs : int = 0;
		for (indexTabs = 0; indexTabs < arrayTabTitles.length; indexTabs++)
		{
		strReturn = strReturn +"		" +  "public var _tabTitles" + (indexTabs + 1) + ":Array = new Array(" + arrayTabTitles[indexTabs] + ");" + "\n";
		strReturn = strReturn + "		" + "[Bindable(\"__NoChangeEvent__\")]"+"\n";
		strReturn = strReturn +"		" +  "public function get tabTitles" + (indexTabs + 1) + "():Array"+"\n";
		strReturn = strReturn + "		" + "{"+"\n";
		strReturn = strReturn +"		" +  "return _tabTitles" + (indexTabs + 1) +";" + "\n";
		strReturn = strReturn +"		" +  "}"+"\n";
		}
		
		strReturn +="		" + 'private var _selectedObject:Object;\n';
		strReturn +="		" + '[Bindable("selectedObject")]\n';
		strReturn +="		" + 'public function get selectedObject():Object\n';
		strReturn +="		" + '{return _selectedObject;}\n';
		strReturn +="		" + 'public function set selectedObject(value:Object):void\n';
		strReturn +="		" + '{\n';
		strReturn +="		" + '   if (value != _selectedObject)\n';
		strReturn +="		" + '	{\n';
		strReturn +="		" + '	    _selectedObject = value;\n';
		strReturn +="		" + '	    dispatchEvent(new Event("selectedObjectChanged"));\n';
		strReturn +="		" + '	}\n';
		strReturn +="		" + '}\n';
		
		strReturn = strReturn + "		" + "]]>"+"\n";
		
		strReturn = strReturn + "</fx:Script>"+"\n";
		
		return strReturn;
		}
		
		private function getDataBindingBreads():Object
		{
		return "<js:beads><js:ViewDataBinding/></js:beads>" ;
		}		
		
		
		
		*/
		
		// not used anymore?
		private function mainViewFileHeader():String
		{
			return "<?xml version=\"1.0\" encoding=\"utf-8\"?> \n<js:View  xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" xmlns:local=\"*\" xmlns:layout=\"layout.*\" xmlns:views=\"views.*\">"
		}
		// not used anymore?
		private function mainViewFileFooter(): String
		{
			return "</js:View>"
		}
		
		
		// header/footer of views
		
		private function mxmlFileHead(_IsInitialView: Boolean): String
		{
			if(_IsInitialView && false)
				return "we use same component for the main view and tabs"
			else
				//return  "<js:Container xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" \n xmlns:layout=\"layout.*\" xmlns:views = \"views.*\">\n";
				return '<mdl:Grid ' +
					'xmlns:fx="http://ns.adobe.com/mxml/2009"\n' +
					'xmlns:js="library://ns.apache.org/flexjs/basic"\n' +
					'xmlns:mdl="library://ns.apache.org/flexjs/mdl"\n' +
					'xmlns="http://www.w3.org/1999/xhtml"' +
					'xmlns:layout="layout.*"' +
					'xmlns:local="local.*"' +
					'xmlns:views="views.*"' +
					'>'
			
		}
		
		private function mxmlFileFoot(_IsInitialView: Boolean): String
		{
			if(_IsInitialView && false)
				return "we use same component for the main view and tabs"
			else
				//return "\n</js:Container>";
				return "\n</mdl:Grid>";
		}
		
		// tab groups
		private function tabGroupHeader( strIDTabGroup:String,arrayTabTitles:Array,maxTabHeightInCurrentGroup:int, width:int ) : String 
		{
			//			var strMXMLData: String = "";
			//			strMXMLData = strMXMLData + "<js:Container  id=\"CTabBar" + strIDTabGroup + "\" height=\"" + (maxTabHeightInCurrentGroup + /*keep room for tab bar since it is larger than bmml*/70) + "\">";
			//			strMXMLData = strMXMLData + "<js:beads><js:OneFlexibleChildVerticalLayout flexibleChild=\"tabbarview" + strIDTabGroup + "\"/></js:beads>";
			//			
			//			strMXMLData = strMXMLData + "<layout:TabBar dataProvider=\"{tabTitles" + (arrayTabTitles.length + 1)/*the tab title is not added to array yet*/  + "}\" width=\"" + width + "\" height=\"50\" id=\"tabbar"+ strIDTabGroup + "\" viewContainer=\"{tabbarview" + strIDTabGroup + "}\" />";
			//			strMXMLData = strMXMLData + "<layout:TabBarView id=\"tabbarview" + strIDTabGroup + "\" width=\"50%\" height=\"50%\" x= \"1\" y=\"30\">";
			//			return strMXMLData;
			
			
			// the strIDTabGroup is tab group's reversed index in the whole project, it doesn't match what we need: the tab group index of current file
			
			var strMXMLData : String = 
				'<mdl:NavigationLayout width="100%">\n' +
				'<!-- ** Note: Santanu **\n' +
				'Dont forgot to use tabbar wrapped inside a Header\n' +
				'else this may not work -->\n' +
				'<mdl:Header>\n' +
				'<mdl:TabBar selectedIndex="0" tabIdField="tabId" labelField="label" className="customTabBarSetup">\n' +
				'<mdl:beads>\n' +
				'<js:ConstantBinding\n' +
				'sourcePropertyName="_tabsLayout' + ( arrayTabTitles + 1) +'"\n' +
				'destinationPropertyName="dataProvider" />\n' +
				'</mdl:beads>\n' +
				'</mdl:TabBar>\n' +
				'</mdl:Header>\n' +
				'<mdl:NavigationLayoutContent>';
			
			return strMXMLData;
		}
		
		
		
		
		private function tabCode(strTABNameNoSpace : String) : String
		{
			//return "<views:"+strTABNameNoSpace+"Tab id=\""+strTABNameNoSpace+"\"/>";
			//			<mdl:TabBarPanel id="targaryens_panel">
			//				<local:TabContentGridC width="100%"/>
			//			</mdl:TabBarPanel>
			
			return '<mdl:TabBarPanel id="' +strTABNameNoSpace + '_panel">\n' +
				'<views:' + strTABNameNoSpace + 'Tab width="100%"/>\n' +
				'</mdl:TabBarPanel>'
		}
		
		
		private function tabGroupFooter(): String
		{
			//return 	 "</layout:TabBarView>\n" + "</js:Container>";
			return '</mdl:NavigationLayoutContent>\n' +
				'</mdl:NavigationLayout>\n' 
		}
		
		private function tabGroupFooter2(): String
		{
			// not sure if it is used at all
			//return  "</layout:TabBarView>\n" + "</js:Container>";
			return tabGroupFooter();
		}
		
		
		// scripts and local variables, used by tab groups and data manager
		
		private function getFXScript(arrayTabTitles:Array, arrayTabIds:Array,  strFieldNames:String, isCustomTAB:Boolean):String
		{
			var strReturn:String = "";
			
			strReturn = strReturn + "<fx:Script>"+"\n";
			strReturn = strReturn + "<![CDATA["+"\n";
			if (isCustomTAB) 
			{
				strReturn = strReturn + "		" + "import events.TabItemClickedEvent;" + "\n";
				strReturn = strReturn + "		" + "import org.apache.flex.events.Event;" + "\n";
				strReturn = strReturn + "		" + "import org.apache.flex.html.SimpleAlert;" + "\n";
			}
			strReturn +="		" + 'import org.apache.flex.events.CustomEvent;\n';
			strReturn +="		" + 'import org.apache.flex.events.Event;\n';
			strReturn +="		" + 'import vos.TabVO;\n';
			strReturn +="		" +  "public var _controls:Array = new Array(" + strFieldNames + ");" + "\n";
			strReturn +="		" +  "[Bindable(\"__NoChangeEvent__\")]"+"\n";
			strReturn +="		" +  "public function get controls():Array"+"\n";
			strReturn +="		" +  "{"+"\n";
			strReturn +="		" +  "    return _controls;" + "\n";
			strReturn +="		" +  "}"+"\n";
			
			var indexTabGroups : int = 0;
			// prominic tab component (will be removed)
			for (indexTabGroups = 0; indexTabGroups < arrayTabTitles.length; indexTabGroups++)
			{
				strReturn = strReturn +"		" +  "public var _tabTitles" + (indexTabGroups + 1) + ":Array = new Array(" + arrayTabTitles[indexTabGroups] + ");" + "\n";
				strReturn = strReturn + "		" + "[Bindable(\"__NoChangeEvent__\")]"+"\n";
				strReturn = strReturn +"		" +  "public function get tabTitles" + (indexTabGroups + 1) + "():Array"+"\n";
				strReturn = strReturn + "		" + "{"+"\n";
				strReturn = strReturn +"		" +  "return _tabTitles" + (indexTabGroups + 1) +";" + "\n";
				strReturn = strReturn +"		" +  "}"+"\n";
			}
			
			// MDL tab
			indexTabGroups  = 0;
			for (indexTabGroups = 0; indexTabGroups < arrayTabTitles.length; indexTabGroups++)
			{
				//				private var _tabsLayout:Array = [
				//					new TabVO("starks_panel", "Starks"),
				//					new TabVO("lannisters_panel", "Lannisters"),
				//					new TabVO("targaryens_panel", "Targaryens")
				//				];
				// The tab titles and ids were not created as Array of strings, but a longer string consists of the ids like "Tab1", "Tab2"
				// I don't want to change existing logic since it needs extra testing and it is easy to restore it to Array of Strings
				// Pan @ Feb 2017
				strReturn +=  "		" +  "public var _tabsLayout" + (indexTabGroups + 1) + ":Array = [";// + arrayTabTitles[indexTabs] + 
				var titlesOfThisGroup = arrayTabTitles[indexTabGroups].split(",");
				var idsOfThisGroup = arrayTabIds[indexTabGroups].split(",");
				for (var indexTab = 0 ; indexTab < titlesOfThisGroup.length; indexTab++)
				{
					strReturn +='new TabVO("' + idsOfThisGroup[indexTab].substring(1, idsOfThisGroup[indexTab].length -1 ) + '_panel", ' + titlesOfThisGroup[indexTab] + ')';
					if (indexTab < titlesOfThisGroup.length - 1)
						strReturn +=',';
					
				}
				strReturn +="];" + "\n\n";
			}
			
			
			strReturn +="		" + 'private var _selectedObject:Object;\n';
			strReturn +="		" + '[Bindable("selectedObject")]\n';
			strReturn +="		" + 'public function get selectedObject():Object\n';
			strReturn +="		" + '{return _selectedObject;}\n';
			strReturn +="		" + 'public function set selectedObject(value:Object):void\n';
			strReturn +="		" + '{\n';
			strReturn +="		" + '   if (value != _selectedObject)\n';
			strReturn +="		" + '	{\n';
			strReturn +="		" + '	    _selectedObject = value;\n';
			strReturn +="		" + '	    dispatchEvent(new Event("selectedObjectChanged"));\n';
			strReturn +="		" + '	}\n';
			strReturn +="		" + '}\n';
			
			strReturn = strReturn + "		" + "]]>"+"\n";
			
			strReturn = strReturn + "</fx:Script>"+"\n";
			
			return strReturn;
		}
		
		private function getDataBindingBreads():String
		{
			//return "<js:beads><js:ViewDataBinding/></js:beads>" ;
			
			return '<mdl:beads><js:ContainerDataBinding/></mdl:beads>';
			
		}		
		
		
		
		private function isWritingSingleDomainProject() : Boolean 
		{
			return fileNameArrays != null && fileNameArrays.length == 1;
		}
		
		
		// main entrance of converting logic
		private function xmlDownloadedMultiComplete(event:Event):void
		{
			try{
				
				
				setTimeout(custom1, 3000);
				function custom1():void {
					xmlData = new XML(event.target.data);
					
					arrXMLData.push(xmlData);
					
					XVarianceAllowed = 10;
					YVarianceAllowed = 10;
					
					//extractControls();
					
					// ADDED by Pan @ 08,NOV
					
					// #number of tab group
					resetTabGroupId();
					
					extractDomainClassInformation();
					
					var isSingle = isWritingSingleDomainProject();
					
					// These two files should be produced only once for all input models
					// moonshineide project file (fixed file content)
					//trace("Export project file");
					
					if (isSingle)
					{
						exportProjectFile(isSingle);
						exportMainViewsOfSingleDomainProject(isSingle);
					} else {
						// TODO out side the loop
					}
					
					
					
					//trace("components checkpoint0=" + arrComponents.length);
					// single object view                
					// each field's meta information is parsed inside the method.
					extractControls();
					
					//trace("components checkpoint1=" + arrComponents.length);
					
					trace("Export single object view");
					exportToFile2();
					//trace("components checkpoint2=" + arrComponents.length);
					
					// domain class
					//trace("Export domain class and others");
					exportDomainClass();
					
					// domain class utils
					//trace("Export domain class utils");
					exportDomainClassUtils();
					
					// data access layer
					//trace("Export data access layer");
					exportDataAccessLayer();
					
					// for MDL 
					exportTableItemRenderer();
					
					
					
					exportObjectListView();
					
					// data access layer (fixed file content)
					//trace("Export data access layer");
					exportDataAccessLayer();
					
					// data access layer (fixed file content)
					//trace("Export Ui utils");
					exportUiUtils();
					
					copyStaticFileTo("FlexJSConfiguration.as", "src/FlexJSConfiguration.as");
					//					copyStaticFileTo("TabBar.as", "src/layout/TabBar.as");
					//					copyStaticFileTo("TabBarView.as", "src/layout/TabBarView.as");
					//					copyStaticFileTo("TabItemClickedEvent.as", "src/events/TabItemClickedEvent.as");
					
					copyStaticFileTo("AutoExpandingTextInput.as", "src/layout/AutoExpandingTextInput.as");
					
					// files used by MDL
					copyStaticFileTo("mdl-styles.css", "src/resources/mdl-styles.css");
					copyStaticFileTo("TabVO.as", "src/vos/TabVO.as");
					copyStaticFileTo("mdl-js-index-template.html", "src/resources/mdl-js-index-template.html");
					
					copyStaticFileTo("googleapi_MaterialIcons.css", "src/resources/googleapi_MaterialIcons.css");
					copyStaticFileTo("material.min.css", "src/resources/material.min.css");
					copyStaticFileTo("material.min.js", "src/resources/material.min.js");
					copyStaticFileTo("CustomTabBarPanelItemRenderer.mxml", "src/itemRenderers/CustomTabBarPanelItemRenderer.mxml");
					
					
					processNextDomain(false);
					
					
					
					// temp files for testing only
					//copyStaticFileTo("Product.as", "src/" + "demo" +"/Product.js");
					//copyStaticFileTo("Product.as", "src/" + getPackageAsFilePath() +"/Product.as");
					//copyStaticFileTo("ProductsModel.as", "src/" + getPackageAsFilePath() +"/ProductsModel.as");
					
				}
			}
			catch (e:Error)
			{
				Alert.show("The XML file \"" + strXMLPath + "\" could not be parsed during processing.");
				blnValidXML = false;
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		
		public function exportToFile2():void
		{
			var strMXMLAll:String = "";
			var  strFormatted:String = "";
			try{
				//var file:File = File.desktopDirectory.resolvePath(strMXMLExportFilePath);
				var objectViewFileName:String = getObjectViewFileName();
				var file:File = File.desktopDirectory.resolvePath(objectViewFileName);
				var stream:FileStream = new FileStream();
				var strWrite:String;
				var intKtr:int;
				
				var strFileName:String;
				var strProjectPath:String;
				var strProjectViewPath:String;
				var strProjectViewFilePath:String;
				var fileView:File;
				
				
				fileView = file;
				
				collectMXMLData();
				
				stream.open(fileView, FileMode.WRITE);
				
				strMXMLAll = getMXMLCode(strMXMLData, true);
				
				
				//				strMXMLAll = mainViewFileHeader() + "\n";
				//				
				//				strMXMLAll = strMXMLAll + strMXMLData;
				//				
				//				strMXMLAll = strMXMLAll + "\n" + mainViewFileFooter();
				
				
				var xml:XML = new XML(strMXMLAll);
				
				xml = moveBeadsToTop(xml);
				
				
				strFormatted = xml.toXMLString();
				
				strFormatted = strReplace(strFormatted, "  ", String.fromCharCode(9));
				stream.writeUTFBytes(strFormatted);
				
				stream.close();
				
				
				var intLen:int;
				
				for (var j:uint = 0;j < arrComponents.length;j++)
				{
					intLen= arrComponents[j].ChildCustomTABsData.length;
					
					for (intKtr = 0; intKtr < intLen; intKtr++)
					{	
						exportNestedTABs(arrComponents[j].ChildCustomTABsData[intKtr]);
					}
				}
				
				
			} // try
			
			catch (e:Error)
			{
				trace(e.getStackTrace()); 
				trace("current xml = ");
				trace(strMXMLAll);
				
				//trace("formatted xml = ");
				//trace(strFormatted);
				Alert.show(e.getStackTrace());
				
			}
		}
		
		private function moveBeadsToTop( mxml : XML) : XML 
		{
			//TODO NOT sure if it is needed for MDL
			var js:Namespace = mxml.namespace("js"); 
			var fx:Namespace = mxml.namespace("fx"); 
			
			if (mxml.js::beads[0] != null)
			{
				var beads:XML = mxml.js::beads[0].copy();
				delete mxml.js::beads[0];
				mxml.insertChildAfter(null, beads);
			}
			
			if (mxml.fx::Script[0] != null)
			{
				var script:XML = mxml.fx::Script[0].copy();
				delete mxml.fx::Script[0];
				mxml.insertChildAfter(null, script);
			}
			return mxml;
		}
		
		
		private function exportNestedTABs(objComponent:Object):void
		{
			
			var strFileXMLData:String;
			var file:File = File.desktopDirectory.resolvePath(strMXMLExportFilePath);
			var stream:FileStream = new FileStream();
			var strWrite:String;
			var intKtr:int;
			
			var strFileName:String;
			var strProjectPath:String;
			var strProjectViewPath:String;
			var strProjectViewFilePath:String;
			var fileView:File;
			var intChildTabs = objComponent.ChildCustomTABsData.length
			var hasNestedTab:Boolean = objComponent.ChildCustomTABsData.length > 0;
			
			strFileName=objComponent.fileName;
			strFileXMLData=objComponent.text;
			
			if(hasNestedTab)
				strWrite = getMXMLCode(strFileXMLData, false);
			else
				strWrite = getMXMLCode(strFileXMLData, false);
			//strWrite = wrapMXMLCode(strFileXMLData);
			
			
			strProjectPath = file.parent.nativePath;
			
			strProjectViewPath = strProjectPath;
			strProjectViewFilePath = strProjectViewPath + "/" + strFileName;
			
			strProjectViewFilePath = getTabViewFileName(strFileName);
			
			fileView = File.desktopDirectory.resolvePath(strProjectViewFilePath);
			
			stream.open(fileView, FileMode.WRITE);
			stream.writeUTFBytes(strWrite);
			
			stream.close();
			
			
			
			for (intKtr = 0; intKtr < intChildTabs; intKtr++)
			{
				exportNestedTABs(objComponent.ChildCustomTABsData[intKtr]);
			}
			
			return;
		}
		
		
		/*
		private function wrapMXMLCode(strCode:String):String
		{
		var strReturn:String;
		try{
		
		strReturn = mxmlFileHeadWithoutTabs();
		
		//				strReturn = strReturn + "<fx:Style>.alignTop{vertical-align: top;}</fx:Style>";
		//				strReturn = strReturn + "<js:Container width=\"100%\" height=\"100%\"  style=\"border:0px solid black\">";
		//				
		//				strReturn = strReturn + "<js:beads><js:VerticalLayout /></js:beads>";
		
		strReturn = strReturn + strCode;
		
		//				strReturn = strReturn + "\n</js:Container> \n </js:Container> \n";						
		strReturn = strReturn + "\n</js:Container>";
		var xml:XML = new XML(strReturn);
		
		strReturn = xml.toXMLString();
		
		strReturn = strReplace(strReturn, "  ", String.fromCharCode(9));
		}
		catch (e:Error)
		{
		trace(e.getStackTrace()); Alert.show(e.getStackTrace());
		}
		return strReturn;
		}
		*/
		
		public function getMXMLCode(strCode:String, IsInitialView:Boolean):String
		{
			try{
				var strMXMLAll:String;
				var xml:XML;
				strMXMLAll = mxmlFileHead(IsInitialView);
				//				if(IsInitialView)
				//					strMXMLAll = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<js:View xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" \n xmlns:layout=\"layout.*\" xmlns:views = \"views.*\">\n";
				//				else
				//					strMXMLAll = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<js:Container xmlns:fx=\"http://ns.adobe.com/mxml/2009\" xmlns:js=\"library://ns.apache.org/flexjs/basic\" \n xmlns:layout=\"layout.*\" xmlns:views = \"views.*\">\n";
				
				strMXMLAll = strMXMLAll + strCode;
				strMXMLAll += mxmlFileFoot(IsInitialView);
				//				if(IsInitialView)
				//					strMXMLAll = strMXMLAll + "\n</js:View>";
				//				else
				//					strMXMLAll = strMXMLAll + "\n</js:Container>";
				
				xml = new XML(strMXMLAll);
				
				//var strWrite:String = strMXMLAll;
				var strWrite:String = xml.toXMLString();
				
				strWrite = strReplace(strWrite, "  ", String.fromCharCode(9));
				
				return strWrite;
			}
			catch (e:Error)
			{
				trace("bad xml = \n");
				trace(strMXMLAll);
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return "";
		}
		
		
		
		// Codes more deeply assosiated with FlexJS syntax
		
		private function isControlButtonBar(strSourceControlTypeID):Boolean
		{
			return false; // MDL doesnt support ButtonBar or similar control
			//return (strSourceControlTypeID == "com.balsamiq.mockups::ButtonBar")
		}
		
		private function isControlTabBar(strSourceControlTypeID):Boolean
		{
			
			return  (strSourceControlTypeID == "com.balsamiq.mockups::TabBar")
			
		}
		
		private function isControlHRule(strSourceControlTypeID):Boolean
		{
			
			return (strSourceControlTypeID == "com.balsamiq.mockups::HRule")
		}
		
		private function isControlNumericStepper(strSourceControlTypeID):Boolean
		{
			
			return (strSourceControlTypeID == "com.balsamiq.mockups::NumericStepper")
			
		}
		
		
		private function isControlNeedsData(strSourceControlTypeID):Boolean
		{
			return (strSourceControlTypeID == "com.balsamiq.mockups::ComboBox" || strSourceControlTypeID == "com.balsamiq.mockups::DropDownList" || /*strSourceControlTypeID == "com.balsamiq.mockups::ButtonBar" ||*/ strSourceControlTypeID == "com.balsamiq.mockups::List")
			
		}
		
		
		
		private function componentMapper(strName:String):String
		{
			try{
				switch (strName)
				{
					case "com.balsamiq.mockups::DateChooser": 
						return "js:DateField";
						break;
					
					case "com.balsamiq.mockups::Image": 
						return "js:Image";
						break;
					
					case "com.balsamiq.mockups::BarChart": 
						return "js:BarChart";
						break;
					
					case "com.balsamiq.mockups::ColumnChart": 
						return "js:ColumnChart";
						break;
					
					case "com.balsamiq.mockups::LineChart": 
						return "js:LineChart";
						break;
					
					case "com.balsamiq.mockups::PieChart": 
						return "js:PieChart";
						break;
					
					case "com.balsamiq.mockups::TabBar": 
						return "js:TabBar";
						break;
					
					case "com.balsamiq.mockups::ButtonBar": 
						//return "js:ButtonBar";
						return "js:Label";
						break;
					
					case "com.balsamiq.mockups::DropDownList": 
						return "js:DropDownList";
						break;
					
					case "com.balsamiq.mockups::ComboBox": 
						return "js:DropDownList";
						break;
					
					case "com.balsamiq.mockups::CheckBox": 
						return "js:CheckBox";
						break;
					case "com.balsamiq.mockups::CheckBoxGroup": 
						return "js:CheckBox";
						break;
					
					case "com.balsamiq.mockups::List": 
						return "js:List";
						break;
					
					case "com.balsamiq.mockups::HRule": 
						return "js:HRule";
						break;
					
					case "com.balsamiq.mockups::VRule": 
						return "js:VRule";
						break;
					
					case "com.balsamiq.mockups::NumericStepper": 
						return "js:NumericStepper";
						break;
					
					case "com.balsamiq.mockups::RadioButton": 
						return "js:RadioButton";
						break;
					
					case "com.balsamiq.mockups::RadioButtonGroup": 
						return "js:RadioButton";
						break;
					
					
					case "com.balsamiq.mockups::TextArea": 
						return "js:TextArea";
						break;
					
					case "com.balsamiq.mockups::TextInput": 
						//return "js:TextInput";
						return "layout:AutoExpandingTextInput";
						break;
					
					case "com.balsamiq.mockups::Button": 
						return "js:TextButton";
						break;
					
					case "com.balsamiq.mockups::Label": 
						return "js:Label";
						break;
					
					case "com.balsamiq.mockups::Title": 
						return "js:Label";
						break;
					
					case "com.balsamiq.mockups::Paragraph": 
						return "js:Label";
						break;
					
					case "com.balsamiq.mockups::BrowserWindow": 
						return "js:Container";
						break;
					
					case "com.balsamiq.mockups::TitleWindow": 
						return "js:Container";
						break;
					
					case "com.balsamiq.mockups::TabBar": 
						return "js:Container";
						break;
					
					case "com.balsamiq.mockups::FieldSet": 
						return "js:Container";
						break;
					
					case "com.balsamiq.mockups::Canvas": 
						return "js:Container";
						break;
					
					//          case "__group__": 
					//              return "js:Container";
					//              break;
				}
				
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return "";
			
		}
		
		
		public function convertControlsInContainer(intParentContainerIndex:int):void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				var innerMostTab:String;
				arrControls.sortOn("AXAY", Array.NUMERIC);
				
				//trace ("arrComponents is reset")
				arrComponents = new Array();
				
				intLen = arrControls.length;
				
				var producedInputControls : Array = new Array();
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					var strSourceControlTypeID = arrControls[intKtr].controlTypeID;
					var strComponentName:String = "";
					var strCSSStyle:String = "";
					
					strComponentName = componentMapper(arrControls[intKtr].controlTypeID);
					
					if (strComponentName != "")
					{
						var strPropertyName:String = "";
						var strFJXML:String = "";
						var intWidth:int = 0;
						var intHeight:int = 0;
						var strPrepareDataProvider:String = "";
						var strID:String = "";
						var strTABName:String = "";
						
						var tograils:String = readParameterInCustomData(arrControls[intKtr].controlProperties.customData.toString(), "ToGrails");
						if (arrControls[intKtr].controlProperties != null 
							&& arrControls[intKtr].controlProperties.FieldName != null
							&& arrControls[intKtr].controlProperties.FieldName.toString().length > 0
							&& 'true' == readParameterInCustomData(arrControls[intKtr].controlProperties.customData.toString(), "ToGrails"))
						{
							//var toGrails: String = readParameterInCustomData(arrControls[intKtr].controlProperties.customData.toString(), "ToGrails");
							
							var GrailNameForThis:String = readParameterInCustomData(arrControls[intKtr].controlProperties.customData.toString(), "GrailsName");
							
							strID = GrailNameForThis;
						} else {
							if(intParentContainerIndex>=0) 
							{
								strID = "id" + intParentContainerIndex.toString() + "_" + intKtr.toString();
								
							}						
							else
								strID = "id" + intKtr.toString();
						}
						
						if (producedInputControls.indexOf(strID) > 0) 
						{
							strID = strID + intKtr.toString();
						}
						producedInputControls.push(strID);
						//trace ("produced:" + producedInputControls);
						
						
						
						strFJXML = "<" + strComponentName;
						
						strPropertyName = propertyMapper(arrControls[intKtr].controlID);
						strFJXML = strFJXML + " id=\"" + strID + "\"";
						
						
						var calculatedText: String = convertTextOrFormula(arrControls[intKtr].text);
						strPropertyName = propertyMapper(calculatedText);
						
						if (strPropertyName != "")
						{
							//if (strComponentName == "js:ComboBox" || strComponentName == "js:DropDownList" || strComponentName == "js:ButtonBar" || strComponentName == "js:List")
							if (isControlNeedsData(strSourceControlTypeID))
								strPrepareDataProvider = prepareDataProvider(strPropertyName);
								//else if (strComponentName == "js:NumericStepper")
							else if (isControlNumericStepper(strSourceControlTypeID))
								strFJXML = strFJXML + " maximum=\"100\" minimum=\"1\" stepSize=\"1\" value=\"" + strPropertyName + "\"";
								//else if (strComponentName == "js:HRule")
							else if(isControlHRule(strSourceControlTypeID)) 
								""
								//else if (strComponentName == "js:TabBar")
							else if(isControlTabBar(strSourceControlTypeID)) 
							{
								strTABName = getTABNameByIndex(strPropertyName, arrControls[intKtr].SelectedIndex);
								strTABName = getUniqueTabName(strTABName);
								innerMostTab = strTABName;
								//trace("---------swith tabname to" + strTABName);
								strFJXML = strFJXML + " text=\"" + strTABName + "\"";
							}
							else
								strFJXML = strFJXML + " text=\"" + htmlEscape(strPropertyName) + "\"";
						}
						
						intWidth = getControlWidth(arrControls[intKtr]);
						
						//if (strComponentName == "js:ButtonBar")
						if (isControlButtonBar(strSourceControlTypeID))
							intWidth = intWidth * 1.8;
						
						strFJXML = strFJXML + " width=\"" + intWidth + "\"";
						
						intHeight = getControlHeight(arrControls[intKtr]);
						
						//if (strComponentName == "js:HRule")
						if (isControlHRule(strSourceControlTypeID))
							intHeight = intHeight/10;
						
						strCSSStyle = getCSSStyle(arrControls[intKtr]);
						
						strFJXML = strFJXML + " height=\"" + intHeight + "\"" + strCSSStyle + " className=\"alignTop\">";
						
						strFJXML = strFJXML + strPrepareDataProvider;
						
						strFJXML = strFJXML + "</" + strComponentName + ">";
						
						strFJXML = unescape(strFJXML);
						
						//////strFJXML = convertGroupControls(strFJXML);
						addControlToArrComponents(strFJXML,strID, strComponentName, strTABName, arrControls[intKtr], intWidth, intHeight);
						
						// update tab information to fields meta
						
						updateFieldsTabInformation(strID/* aka grails name*/,innerMostTab );
					}
				}
			}
			catch (e:Error)
			{
				
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			
			trace("convertControlsInContainer, controls=" + arrControls.length + " arrComponents=" + arrComponents.length);
		}
		
		
		
		
		private function addControlToArrComponents(strFJXML: String,strID:String, strComponentName: String, strTABName: String, arrControl:Object, intWidth:int, intHeight:int): void
		{
			////arrComponents.push({text: strFJXML, controlTypeID: arrControls[intKtr].controlTypeID, x: arrControls[intKtr].x, y: arrControls[intKtr].y, XY: arrControls[intKtr].XY, w: intWidth, h: intHeight, isInGroup: arrControls[intKtr].isInGroup, locked: arrControls[intKtr].locked, controlID: arrControls[intKtr].controlID, size: arrControls[intKtr].size, color: arrControls[intKtr].color, align: arrControls[intKtr].align});
			var optionList:String = readParameterInCustomData(arrControl.controlProperties.customData.toString(), "OptionList")
			var i : int = 0;
			var label:String;
			var value:String;
			var options: Array;
			var count: int;
			if (strComponentName == 'js:CheckBox')
			{
				if (optionList == null)
				{
					count = 0;
				} else {
					
					options = optionList.split(",");
					count = options.length;
					
				}
				
				var checkBoxFromBoxesToArray:String = strID + ".text='';";
				var checkBoxFromArrayToBoxes:String = "";
				for (i = 0; i < count; i++)
				{
					if (options[i].indexOf("|") > 0) 
					{
						value = options[i].split("|")[1];
					} else {
						value = options[i] ;
					}
					
					checkBoxFromBoxesToArray += "if ("+ strID + "_" + value + ".selected) " + strID + ".text+='" + value + "'+';';" ;
					//checkBoxFromArrayToBoxes +=  strID + "_" + value + ".selected=" +strID + ".text.indexOf('" + value + ";') > -1;";
					checkBoxFromArrayToBoxes +=  strID + "_" + value + ".selected=" +strID + ".text.split(';').indexOf('" + value + "') > -1;";
				}
				
				checkBoxFromBoxesToArray +='if ( ' + strID + ".text.lastIndexOf(';') == " + strID + '.text.length - 1) ' + strID +  '.text = ' + strID + '.text.substr(0, '+ strID + '.text.length - 1);';
				for (i = 0; i < count; i++)
				{
					//trace("~~~~~~~~~~~optionlist:" + strComponentName + ":" + options[i]);
					
					
					if (options[i].indexOf("|") > 0) 
					{
						label = options[i].split("|")[0];
						value = options[i].split("|")[1];
					} else {
						label = options[i];
						value =  options[i]  ;
					}
					
					// TODO replace text
					strFJXML = strFJXML.replace(/text=".*" /gis, 'text="' + label + '" ');
					strFJXML = strFJXML.replace(/id=".*" /gi, 'id="' +strID + "_" + value +'" text="' + label+'" change="' + checkBoxFromBoxesToArray +'" ');
					
					// it has no value
					if (i == count -1 ) {
						strFJXML +='\n <js:TextInput id="' + strID + '" text="" visible="false" change="' + checkBoxFromArrayToBoxes + '"/>\n'
					}
					arrComponents.push({bmmlControl: arrControl, text: strFJXML, controlTypeID: arrControl.controlTypeID, x: arrControl.x, y: arrControl.y, XY: arrControl.XY, w: intWidth, h: intHeight, isInGroup: arrControl.isInGroup, locked: arrControl.locked, controlID: arrControl.controlID, size: arrControl.size, color: arrControl.color, align: arrControl.align, IsCustomTAB: false, TABName: strTABName, IsChildCustomTAB: false, ChildCustomTABsData: []});
				}
				
			} else if (strComponentName == 'js:RadioButton')
			{
				if (optionList == null)
				{
					count = 0;
				} else {
					
					options = optionList.split(",");
					count = options.length;
					
				}
				
				var radioButtonActualValueFieldOnChange:String = "";
				for (i = 0; i < count; i++)
				{
					if (options[i].indexOf("|") > 0) 
					{
						
						value = options[i].split("|")[1];
					} else {
						value = options[i]  ;
					}
					
					radioButtonActualValueFieldOnChange += "if ("+ strID + ".text=='" + value + "') " + strID + "_" + value + ".selected='true';" ;
				}
				//radioButtonActualValueFieldOnChange = 'ooooo';
				for (i = 0; i < count; i++)
				{
					//trace("~~~~~~~~~~~optionlist:" + strComponentName + ":" + options[i]);
					if (options[i].indexOf("|") > 0) 
					{
						label = options[i].split("|")[0];
						value = options[i].split("|")[1];
						// TODO replace text
					} else {
						label = options[i];
						value = options[i];
					}
					
					strFJXML = strFJXML.replace(/text=".*" /gis, 'text="' + label + '" ');
					strFJXML = strFJXML.replace(/id=".*" /gi, 'id="' + strID + "_" + value +'" groupName="' + strID +'" text="' + label + '" value="' + value +'" change="if (' + strID + '.text!=\'' +  value + '\') ' + strID +'.text=\'' +  value + '\'" ');
					//strFJXML = strFJXML.replace(/id=".*" /gi, 'id="' + strID + "_" + value +'" groupName="' + strID +'" text="' + label + '" value="' + value +'" change="' + strID +'.text=\'aaaa\'" ');
					
					
					if (i == count -1 ) {
						strFJXML +='\n <js:TextInput id="' + strID + '" text="" visible="false" change="' + radioButtonActualValueFieldOnChange + '"/>\n'
					}
					arrComponents.push({bmmlControl: arrControl,text: strFJXML, controlTypeID: arrControl.controlTypeID, x: arrControl.x, y: arrControl.y, XY: arrControl.XY, w: intWidth, h: intHeight, isInGroup: arrControl.isInGroup, locked: arrControl.locked, controlID: arrControl.controlID, size: arrControl.size, color: arrControl.color, align: arrControl.align, IsCustomTAB: false, TABName: strTABName, IsChildCustomTAB: false, ChildCustomTABsData: []});
				}
				
			} else {
				arrComponents.push({bmmlControl: arrControl,text: strFJXML, controlTypeID: arrControl.controlTypeID, x: arrControl.x, y: arrControl.y, XY: arrControl.XY, w: intWidth, h: intHeight, isInGroup: arrControl.isInGroup, locked: arrControl.locked, controlID: arrControl.controlID, size: arrControl.size, color: arrControl.color, align: arrControl.align, IsCustomTAB: false, TABName: strTABName, IsChildCustomTAB: false, ChildCustomTABsData: []});
			}
			
			//arrComponents.push({bmmlControl: arrControl,text: strFJXML, controlTypeID: arrControl.controlTypeID, x: arrControl.x, y: arrControl.y, XY: arrControl.XY, w: intWidth, h: intHeight, isInGroup: arrControl.isInGroup, locked: arrControl.locked, controlID: arrControl.controlID, size: arrControl.size, color: arrControl.color, align: arrControl.align, IsCustomTAB: false, TABName: strTABName, IsChildCustomTAB: false, ChildCustomTABsData: []});
			
		}
		
		
		
		public function prepareDataProvider(strData:String):String
		{
			try{
				var strDataProvider:String;
				var arrData:Array;
				
				var intLen:int = 0;
				var intKtr:int = 0;
				
				if (strData.indexOf(",") >= 0)
					arrData = strData.split(",");
				else if (strData.indexOf(String.fromCharCode(13)) >= 0)
					arrData = arrData.split(String.fromCharCode(13));
				else
					arrData = strData.split(String.fromCharCode(10));
				
				intLen = arrData.length;
				
				strDataProvider = "<js:dataProvider><fx:Array>";
				
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					strDataProvider = strDataProvider + "<fx:String>" + htmlEscape(arrData[intKtr]) + "</fx:String>";
				}
				
				strDataProvider = strDataProvider + "</fx:Array></js:dataProvider>";
				
				return strDataProvider;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return "";
		}
		
		
		
		
		private function getCSSStyle(objControl:Object):String
		{
			try{
				
				var strStyle:String = "";
				
				if (objControl.controlProperties.bold != null && objControl.controlProperties.bold=="true")
					strStyle = strStyle + "font-weight:bold;";
				
				if (objControl.controlProperties.italic != null && objControl.controlProperties.italic=="true")
					strStyle = strStyle + "font-style:italic;";
				
				if (objControl.controlProperties.underline != null && objControl.controlProperties.underline=="true")
					strStyle = strStyle + "text-decoration:underline;";
				
				
				if (objControl.color != "")
				{
					
					var hexint:String = parseInt(objControl.color).toString(16); 
					if (hexint.length < 6) {
						var oldLength: int = hexint.length;
						for (var i:int= 0;i< 6 - oldLength;i++)
						{
							hexint =  new String("0") + hexint;
						}
					}
					strStyle = strStyle + "color:'#" + hexint + "';";
				}
				
				
				
				if (objControl.size != "")
					strStyle = strStyle + "fontSize:" + objControl.size + ";";
				
				if (objControl.align != "")
					strStyle = strStyle + "textAlign:'" + objControl.align + "';";
				
				
				if (strStyle != "")
				{
					strStyle = strStyle.slice(0, -1);
					
					strStyle = " style=\"" + strStyle + "\" "
				}
				
				return strStyle;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return "";
		}
		
		
		var maxTabHeightInCurrentGroup:int = 0;
		
		
		public function assembleControlsInContainer(intParentContainerIndex:int):void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				var strParentContainerHeader:String = "";
				var strParentContainerFooter:String = "";
				var strContainerHeader:String = "";
				var strContainerFooter:String = "";
				var intLastY:int = 0;
				var intLastX:int = 0;
				var intLastW:int = 0;
				var intLastH:int = 0;
				var intLastMaxH:int = 0;
				var intLastMaxY:int = 0;
				var intContainerX:int = 0;
				var intContainerY:int = 0;
				var blnFirstControl:Boolean = true;
				var blnFirstCustomTAB:Boolean = true;
				var intHCount:int=0;
				var intMarginH:int = 0;
				//var strTABTitles:String = "";
				//var strTABIds:String = "";
				var arrayTabTitles: Array = new Array();
				var arrayTabIds: Array = new Array();
				trace("assembleControlsInContainer index=" +  arrComponents[intParentContainerIndex].controlID);
				var currentTabGroupIndex : int = 0;
				
				var strFieldNames:String = "";
				//var strFieldNamesNotInTab : String = "";
				//var arrayFieldNames: Array = new Array();
				var IsTabBarViewTagOpen:Boolean = false;
				
				
				
				strContainerHeader = "<js:Container style=\"border:0px solid black\" width=\"100%\"><js:beads><js:HorizontalLayout /></js:beads>"
				if (intParentContainerIndex >= 0)
				{
					var strCSSStyle:String = getCSSStyle(arrControls[intKtr]);
					// Need to append strCSSStyle in ParentContainerHeader. The style attributes in this are being lost otherwise
					var isTopView:Boolean = arrComponents[intParentContainerIndex].controlID == 1;
					if (isTopView)
					{
						strParentContainerHeader = "<js:Container height=\"" + "100%" + "\" width=\"" + arrComponents[intParentContainerIndex].w + "\" style=\"border:0px solid gold\"><js:beads><js:VerticalLayout /></js:beads>"
						
					} else {
						
						strParentContainerHeader = "<js:Container height=\"" + arrComponents[intParentContainerIndex].h + "\" width=\"" + arrComponents[intParentContainerIndex].w + "\" style=\"border:0px solid gold\"><js:beads><js:VerticalLayout /></js:beads>"
					}
					
					strParentContainerFooter = "</js:Container>"
				}
				strContainerFooter = "</js:Container>"
				
				if (intParentContainerIndex >= 0)
				{
					intContainerX = arrComponents[intParentContainerIndex].x;
					intContainerY = arrComponents[intParentContainerIndex].y;
				}
				
				intLen = arrComponents.length;
				
				strMXMLData = "";
				
				
				var isTopTap : Boolean  = false;
				
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					var intSpacerW:int = 0;
					var intSpacerH:int = 0;
					var intInlineBlock:int = 4;     // To adjust the extra space due to Display:Inline-Block;
					var strSize:String = "";
					var strColor:String = "";
					var strAlign:String = "";
					var strTABName:String = "";
					var IsCustomTAB:Boolean = arrComponents[intKtr].IsCustomTAB;
					
					//strFieldNames+= arrComponents[intKtr].bmmlControl.controlProperties.FieldName + ",";
					//				if (arrComponents[intKtr].bmmlControl.name == "control" && arrComponents[intKtr].bmmlControl.controlProperties != null && arrComponents[intKtr].bmmlControl.controlProperties.FieldName != null && arrComponents[intKtr].bmmlControl.controlProperties.FieldName.toString().length>0 )
					var isChild:Boolean = isChildOfContainer(arrComponents[intParentContainerIndex], arrComponents[intKtr]); 
					if (isChild && arrComponents[intKtr].bmmlControl.controlProperties != null && arrComponents[intKtr].bmmlControl.controlProperties.FieldName != null && arrComponents[intKtr].bmmlControl.controlProperties.FieldName.toString().length>0)
					{
						if (strFieldNames.length==0)
						{
							strFieldNames+= '"' + arrComponents[intKtr].bmmlControl.controlProperties.FieldName + '"';	
						} else {
							strFieldNames+= ',"' + arrComponents[intKtr].bmmlControl.controlProperties.FieldName + '"';	
						}
					}
					
					
					if (intParentContainerIndex != intKtr && !IsCustomTAB)
					{
						var intDiff:int = arrComponents[intKtr].y - (intLastY + intContainerY);
						
						strSize = arrComponents[intKtr].size;
						strColor = arrComponents[intKtr].color;
						strAlign = arrComponents[intKtr].align;
						
						if (isChild)
						{
							if (intDiff < 0)
								intDiff = intDiff * -1;
							
							if (blnFirstControl)
							{
								intHCount = intHCount + 1;
								intSpacerH = arrComponents[intKtr].y - intLastY - intContainerY - intInlineBlock;
								strMXMLData = strMXMLData + strParentContainerHeader ;
								if (intSpacerH > 0)
									strMXMLData = strMXMLData + "\n" + "<js:Spacer style=\"border:0px solid black\" height=\"" + intSpacerH + "\" width=\"100%\"/>";
								
								strMXMLData = strMXMLData + strContainerHeader;
								
								blnFirstControl = false;
							}
							else if ( (intLastY + intLastH) <= (arrComponents[intKtr].y - intContainerY) || intLastX >= arrComponents[intKtr].x)
							{
								intLastH = 0;
								intLastW = 0;
								intLastX = 0;
								intSpacerH = arrComponents[intKtr].y - intLastY - intLastMaxH - intContainerY - intInlineBlock;
								
								if (IsTabBarViewTagOpen)
								{
									trace("end tab group in IsTabBarViewTagOpen");
									
									trace("tabTitles=" + arrayTabTitles[arrayTabTitles.length - 1]);
									trace("tabids=" + arrayTabIds[arrayTabIds.length - 1]);
									
									
									maxTabHeightInCurrentGroup = 0;
									
									
									strMXMLData += tabGroupFooter2();
									//strMXMLData = strMXMLData + '<js:Container id="case2"/>';
									IsTabBarViewTagOpen = false;
									
									blnFirstCustomTAB = true;
								} else {
									if (intSpacerH > 0)
										strMXMLData = strMXMLData + "\n" + "<js:Spacer  style=\"border:0px solid blue\" height=\"" + intSpacerH + "\" width=\"100%\"/>";
									
								}
								
								
								intLastMaxH = 0;
								intLastMaxY = 0;
								intMarginH = 0;
								
								strMXMLData = strMXMLData + strContainerFooter;
								strMXMLData = strMXMLData + strContainerHeader;
							}
							else
							{
								// Instead of Spacer, need to leave margin here.
								intHCount = intHCount + 1;
								intMarginH = intMarginH + parseInt(arrComponents[intKtr].y) - intLastY - intContainerY;
								
								if (intMarginH > 0)
									applyMargin(intKtr, intMarginH, 0, 0, 0);
								
								//  strMXMLData = strMXMLData + "\n" + "<js:Spacer style=\"border:0px solid blue\" height=\"" + intSpacerH + "\" width=\"100%\"/>";
							}
							
							intSpacerW = arrComponents[intKtr].x - intLastX - intLastW - intContainerX;
							
							if (intSpacerW > 0)
								strMXMLData = strMXMLData + "\n" + "<js:Spacer     style=\"border:0px solid yellow\" width=\"" + intSpacerW + "\" height=\"1\"/>"
							
							strMXMLData = strMXMLData + "\n" + arrComponents[intKtr].text;
							intLastY = arrComponents[intKtr].y - intContainerY;
							intLastX = arrComponents[intKtr].x - intContainerX;
							intLastW = arrComponents[intKtr].w;
							intLastH = arrComponents[intKtr].h;
							
							if (arrComponents[intKtr].y > intLastMaxY)
								intLastMaxY = arrComponents[intKtr].y;
							
							if (arrComponents[intKtr].h > intLastMaxH)
								intLastMaxH = arrComponents[intKtr].h;
							
							arrComponents[intKtr] = "";
						}// end if isChild
					}//end if 
					
					if (IsCustomTAB && isChild)
					{
						//trace ("!!!!!!!!!!!!!!!!! tab goes here: blnFirstCustomTAB=" + blnFirstCustomTAB);
						var strTABXMLData:String;
						var strTABNameNoSpace:String;
						
						var intLengthCustomTabs:int;
						var objChildCustomTAB:Object = {"fileName": "", "TABName": "", "text": "", "ChildCustomTABsData":""};
						
						intLengthCustomTabs = arrComponents[intParentContainerIndex].ChildCustomTABsData.length;
						strTABName = arrComponents[intKtr].TABName;
						strTABNameNoSpace = strTABName.replace(" ", "").replace(" ", "").replace(" ", "").replace(" ", "").replace(" ", "").replace(" ", "").replace("-", "").replace("-", "").replace("\\", "").replace("/", "").replace("\\", "").replace("/", "").replace(".", "").replace("&", "") ;
						arrComponents[intParentContainerIndex].IsChildCustomTAB = true;
						objChildCustomTAB.TABName = strTABName;
						objChildCustomTAB.fileName = strTABNameNoSpace+"Tab.mxml";
						
						// Here need to check if this component is customTab. If its a custom tab then need to integrate the child tabs also here by some means.
						// We need to create some child node in arrComponents to store the details.
						strTABXMLData = arrComponents[intKtr].text;
						
						objChildCustomTAB.text = strTABXMLData;
						
						objChildCustomTAB.ChildCustomTABsData = arrComponents[intKtr].ChildCustomTABsData;
						
						//trace("!!!!!!!!!!!!!!!!! tab added to " +  intLengthCustomTabs + " of arrComponents#" + intParentContainerIndex);
						arrComponents[intParentContainerIndex].ChildCustomTABsData[intLengthCustomTabs] = objChildCustomTAB;
						
						
						if (blnFirstCustomTAB)
						{
							var strIDTabGroup:String;
							strIDTabGroup = getNewTabGroupId();
							
							
							
							trace("create tabbar head");
							
							isTopTap = !blnFirstControl;
							//							if (!blnFirstControl)    /// Atin. Added this line while Pan is changing code
							//							{
							//								//strMXMLData = strMXMLData + "</js:Container>";
							//								strMXMLData = strMXMLData + "<js:Container  id=\"CTabBar" + strIDTabGroup + "\" height=\"" + (maxTabHeightInCurrentGroup + /*keep room for tab bar since it is larger than bmml*/70) + "\">";
							//								// OneFlexibleChildVerticalLayout should be inside "CTabBar". by Pan @ NOV 23
							//								strMXMLData = strMXMLData + "<js:beads><js:OneFlexibleChildVerticalLayout flexibleChild=\"tabbarview" + strIDTabGroup + "\"/></js:beads>";
							//								
							//								isTopTap = true;
							//							} else {
							//								//height=\"100%\"
							//								strMXMLData = strMXMLData + "<js:Container  id=\"CTabBar" + strIDTabGroup + "\" height=\"" + (maxTabHeightInCurrentGroup + /*keep room for tab bar since it is larger than bmml*/70 ) + "\">";
							//								// OneFlexibleChildVerticalLayout should be inside "CTabBar". by Pan @ NOV 23
							//								
							//								strMXMLData = strMXMLData + "<js:beads><js:OneFlexibleChildVerticalLayout flexibleChild=\"tabbarview" + strIDTabGroup + "\"/></js:beads>";
							//								//strMXMLData = strMXMLData + '<js:Container name="case1Iwanttoinserttabhead"/>';
							//								
							//								isTopTap = false;
							//							}
							
							strMXMLData += tabGroupHeader( strIDTabGroup ,arrayTabTitles , maxTabHeightInCurrentGroup, parseInt(arrComponents[intKtr].w));
							
							
							
							
							//strTABTitles = "\"" + strTABName + "\"";
							//strTABIds = "\"" + strTABNameNoSpace + "\"";
							arrayTabTitles.push("\"" + strTABName + "\"");
							arrayTabIds.push("\"" + strTABNameNoSpace + "\"");
							IsTabBarViewTagOpen = true;
							currentTabGroupIndex ++;
						}
						else
						{
							arrayTabTitles[arrayTabTitles.length - 1]  += "," + "\"" + strTABName + "\"";
							arrayTabIds[arrayTabIds.length - 1] += "," + "\"" + strTABNameNoSpace + "\"";
							//strTABTitles = strTABTitles + "," + "\"" + strTABName + "\"";
							//strTABIds = strTABIds + "," + "\"" + strTABNameNoSpace + "\"";
						}
						
						
						//strMXMLData = strMXMLData + "<views:"+strTABNameNoSpace+"Tab id=\""+strTABNameNoSpace+"\" name=\"" + strTABName + "\"  />";
						//FlexJS 0.8(dev) removed the name property
						trace("tab = " + strTABNameNoSpace);
						//strMXMLData = strMXMLData + "<views:"+strTABNameNoSpace+"Tab id=\""+strTABNameNoSpace+"\"/>";
						strMXMLData += tabCode( strTABNameNoSpace);
						
						
						
						//trace("^^^^^^^^ not first tab!");
						blnFirstCustomTAB = false;
						arrComponents[intKtr] = "";
					} else {
						
					}
					
					
				}
				
				
				if (arrComponents[intParentContainerIndex].IsChildCustomTAB && IsTabBarViewTagOpen)
				{
					trace("end tab group in arrComponents[intParentContainerIndex].IsChildCustomTAB && IsTabBarViewTagOpen");
					trace("tabTitles=" + arrayTabTitles[arrayTabTitles.length - 1]);
					trace("tabids=" + arrayTabIds[arrayTabIds.length - 1]);
					strMXMLData += tabGroupFooter();
					
					
					//					strMXMLData = strMXMLData + '<js:Container id="case1"/>';
					IsTabBarViewTagOpen = false;
					maxTabHeightInCurrentGroup = 0;
					
					//blnFirstCustomTAB = true;
				}
				
				
				if(blnFirstControl==false)
					strMXMLData = strMXMLData + strContainerFooter + strParentContainerFooter;
				
				if (intParentContainerIndex >= 0)
				{
					var controlTypeID:String = arrComponents[intParentContainerIndex].controlTypeID;
					//trace("checking Tabbar:" + arrComponents[intParentContainerIndex].controlID + " tabname=" + arrComponents[intParentContainerIndex].TABName);
					if (controlTypeID == "com.balsamiq.mockups::TabBar")
					{
						arrComponents[intParentContainerIndex].controlTypeID = "customTab";
						arrComponents[intParentContainerIndex].IsCustomTAB = true;
						//trace("arrComponents " +  intParentContainerIndex + "is tabbar!");
						
						
						if (arrComponents[intParentContainerIndex].h >  maxTabHeightInCurrentGroup)
						{
							maxTabHeightInCurrentGroup = arrComponents[intParentContainerIndex].h;
						}
						
						//trace("tabbar height =" + arrComponents[intParentContainerIndex].h);
					}
					else
						arrComponents[intParentContainerIndex].controlTypeID = "custom";
					
					
					
					//trace("!!!! producing script:" + arrComponents[intParentContainerIndex].controlID + "name=" + arrComponents[intParentContainerIndex].controlTypeID +  " tabname=" + arrComponents[intParentContainerIndex].TABName + " controls=" + strFieldNames.length + " tabs=" + strTABNames.length );
					if (arrComponents[intParentContainerIndex].IsChildCustomTAB/* && arrComponents[intParentContainerIndex].controlTypeID != "custom"*/)
					{
						var strFXScript:String;
						//trace ("to produce tab scripts");
						
						//strFXScript = getFXScript((arrayTabTitles[arrayTabTitles.length - 1] as String), (arrayTabIds[arrayTabIds.length - 1] as String), strFieldNames, true);
						strFXScript = getFXScript(arrayTabTitles, arrayTabIds, strFieldNames, true);
						var strDataBindingBreads:String = getDataBindingBreads();
						
						//					strMXMLData = strFXScript + "<js:beads><js:ViewDataBinding/><js:OneFlexibleChildVerticalLayout flexibleChild=\"tabbarview_"+intParentContainerIndex+"\" /></js:beads>" + strMXMLData;
						// Fixed for tab views, flexibleChild has to be the id of tabview
						
						var strIDTabGroup:String;
						strIDTabGroup = getCurrentTabGroupId();
						
						if (isTopTap)
						{
							strMXMLData = strFXScript + strDataBindingBreads + strMXMLData;	
							// FlexJS works unpredicatable if this beads is not at the top of the container
							//strMXMLData = insertToTopOfView(strMXMLData, "<js:beads><js:ViewDataBinding/></js:beads>");
						} else {
							strMXMLData = strFXScript + "<js:beads><js:ViewDataBinding/></js:beads>" + strMXMLData;	
							//						strMXMLData = strFXScript + "<js:beads><js:ViewDataBinding/><js:OneFlexibleChildVerticalLayout flexibleChild=\"tabbarview" +strIDTabGroup + "\"/></js:beads>" + strMXMLData;
						}
						
					} else {
						if (arrComponents[intParentContainerIndex].controlTypeID == 'customTab' || arrComponents[intParentContainerIndex].controlID == '1') {
							strFXScript = getFXScript(arrayTabTitles, arrayTabIds, strFieldNames, false);
							
							strMXMLData = strFXScript + strMXMLData;
						}
					}
					
					arrComponents[intParentContainerIndex].text = strMXMLData;
				}
				
				arrComponents = trimArray(arrComponents);
				
				
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		
		
		
		// Codes related to output format ends here
		
		
		// Util functions to work with our bmml files
		
		
		private function getControlWidth(ObjControl:Object):int
		{
			try{
				var intWidth:int = 0;
				var intW:int = 0;
				var intMeasuredW:int = 0;
				var blnContainerControl:Boolean = false;
				var strControlName:String = ObjControl.controlTypeID.toString();
				
				blnContainerControl = isContainerControl(strControlName);
				
				intW = parseInt(ObjControl.w);
				intMeasuredW = parseInt(ObjControl.measuredW);
				
				if (ObjControl.w != "-1")
				{
					// Here we need to calculate the width based on the control type. 
					// For some controls, we need to take min width
					// and for container controls we need to take max width
					var intMinWidth:int = 0;
					var intMaxWidth:int = 0;
					
					if (intW > intMeasuredW)
					{
						intMinWidth = intMeasuredW;
						intMaxWidth = intW;
					}
					else
					{
						intMinWidth = intW;
						intMaxWidth = intMeasuredW;
					}
					
					if(blnContainerControl ||  strControlName == "com.balsamiq.mockups::HRule" || strControlName == "com.balsamiq.mockups::ComboBox")
						intWidth = intMaxWidth;
					else
						intWidth = intMinWidth;
				}
				else
					intWidth = intMeasuredW;
				
				return intWidth;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return -1;
		}
		
		private function getControlHeight(ObjControl:Object):int
		{
			try{
				var intHeight:int = 0;
				var intH:int = 0;
				var intMeasuredH:int = 0;
				var blnContainerControl:Boolean = false;
				
				blnContainerControl = isContainerControl(ObjControl.controlTypeID.toString());
				
				//trace("blnContainerControl : " + blnContainerControl);
				
				intH = parseInt(ObjControl.h);
				intMeasuredH = parseInt(ObjControl.measuredH);
				
				if(intH==0)
					intH = intMeasuredH;
				
				if(intMeasuredH==0)
					intMeasuredH = intH;
				
				
				//trace("intH : " + intH );
				//trace("intMeasuredH : " + intMeasuredH );
				
				if (intH>0)
				{
					// Here we need to calculate the Height based on the control type. 
					// For some controls, we need to take min Height
					// and for container controls we need to take max Height
					var intMinHeight:int = 0;
					var intMaxHeight:int = 0;
					
					if (intH > intMeasuredH)
					{
						intMinHeight = intMeasuredH;
						intMaxHeight = intH;
					}
					else
					{
						intMinHeight = intH;
						intMaxHeight = intMeasuredH;
					}
					
					if(blnContainerControl)
						intHeight = intMaxHeight;
					else
						intHeight = intMinHeight;
				}
				else
					intHeight = intMeasuredH;
				
				//trace("intHeight : " + intHeight );
				
				
				return intHeight;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return -1;
		}
		
		public function htmlEscape(str:String):String {
			try{
				var strReturn:String = "";
				
				strReturn = XML( new XMLNode( XMLNodeType.TEXT_NODE, str ) ).toXMLString();
				
				strReturn = strReturn.split("\"").join ("&quot;");
				
				strReturn = strReturn.replace(/</gi, "&lt;");
				strReturn = strReturn.replace(/>/gi, "&gt;");
				
				
				return strReturn;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return "";
		}
		
		
		
		public function resetXCoordinates():void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				var intLastX:int;
				var intLastAX:int;
				
				
				intLen = arrControls.length;
				
				if (intLen > 0)
				{
					arrControls.sortOn("x", Array.NUMERIC);
					
					intLastX = arrControls[0].x;
					
					for (intKtr = 1; intKtr < intLen; intKtr++)
					{
						arrControls[intKtr].ax = arrControls[intKtr].x;
						
						if (arrControls[intKtr].x - intLastAX < XVarianceAllowed)
						{
							arrControls[intKtr].ax = intLastAX;
						}
						
						intLastX = arrControls[intKtr].x
						intLastAX = arrControls[intKtr].ax
					}
					
				}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		public function resetYCoordinates():void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				var intLastY:int;
				var intLastAY:int;
				
				intLen = arrControls.length;
				
				if (intLen > 0)
				{
					arrControls.sortOn("y", Array.NUMERIC);
					
					
					intLastY = arrControls[0].y;
					
					
					for (intKtr = 1; intKtr < intLen; intKtr++)
					{
						arrControls[intKtr].ay = arrControls[intKtr].y;
						if (arrControls[intKtr].y - intLastAY < YVarianceAllowed)
						{
							arrControls[intKtr].ay = intLastAY;
						}
						
						intLastY = arrControls[intKtr].y;
						intLastAY = arrControls[intKtr].ay;
					}
				}
				
				
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		public function resetXYCoordinates():void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				
				intLen = arrControls.length;
				
				for (intKtr = 1; intKtr < intLen; intKtr++)
				{
					arrControls[intKtr].XY = Number(arrControls[intKtr].y) * 1000 + Number(arrControls[intKtr].x);
				}
				
				arrControls.sortOn("XY", Array.NUMERIC);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		public function resetAXAYCoordinates():void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				
				intLen = arrControls.length;
				
				for (intKtr = 1; intKtr < intLen; intKtr++)
				{
					arrControls[intKtr].AXAY = Number(arrControls[intKtr].ay) * 1000 + Number(arrControls[intKtr].ax);
				}
				
				arrControls.sortOn("AXAY", Array.NUMERIC);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		public function resetXHYHCoordinates():void
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				
				intLen = arrControls.length;
				
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					var intWidth:int = 0;
					var intHeight:int = 0;
					
					if (!isContainerControl(arrControls[intKtr].controlTypeID.toString()))
					{
						if (arrControls[intKtr].w != "-1")
							intWidth = Number(arrControls[intKtr].w);
						else
							intWidth = Number(arrControls[intKtr].measuredW);
						
						if (arrControls[intKtr].h != "-1")
							intHeight = arrControls[intKtr].h;
						else
							intHeight = arrControls[intKtr].measuredH;
					}
					
					arrControls[intKtr].XHYH = (Number(arrControls[intKtr].y) + intHeight) * 1000 + (Number(arrControls[intKtr].x) + intWidth);
				}
				
				arrControls.sortOn("XHYH", Array.NUMERIC);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		private function strReplace(str:String, search:String, replace:String):String
		{
			return str.split(search).join(replace);
		}
		
		public function extractControls():void
		{
			try{
				var intParentContainerIndex:int;
				
				intParentContainerIndex = -1;
				//extractAllControls();
				extractAllControlsArray();
				resetXCoordinates();
				resetYCoordinates();
				resetAXAYCoordinates();
				//resetXHYHCoordinates();
				
				//trace("components checkpoint0.1=" + arrComponents.length);
				
				convertControlsInContainer(intParentContainerIndex);
				intParentContainerIndex = getInnerMostContainer();
				
				do
				{
					//convertControlsInContainer(intParentContainerIndex);
					//trace("before assemble! index=" + intParentContainerIndex + "id=" + arrComponents[intParentContainerIndex].controlID + " controlTypeID=" + arrComponents[intParentContainerIndex].controlTypeID);
					//trace("components checkpoint0.5=" + arrComponents.length);
					//Alert.show("components checkpoint0.5=" + arrComponents.length, "Color Selection", Alert.YES, this);
					assembleControlsInContainer(intParentContainerIndex);
					//trace("components checkpoint0.6=" + arrComponents.length);
					if (arrComponents.length < 10)
					{
						var i : int = 0;
						for (i = 0; i < arrComponents.length; i++)
						{
							//trace ("component #" + i + ".id=" + arrComponents[i].controlID + " type=" + arrComponents[i].controlTypeID); 
						}
					}
					intParentContainerIndex = getInnerMostContainer();
					
					//break;
				} while (intParentContainerIndex != -1);
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		public function getInnerMostContainer():int
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				var intPos:int = -1;
				var intLastContainerX:int;
				var intLastContainerY:int;
				
				//arrControls.sortOn("XY", Array.NUMERIC);
				
				intLen = arrComponents.length;
				
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					var intX:Number = 0;
					var intY:Number = 0;
					var blnContainerControl:Boolean = false;
					
					intX = arrComponents[intKtr].x;
					intY = arrComponents[intKtr].x;
					
					blnContainerControl = isContainerControl(arrComponents[intKtr].controlTypeID);
					
					if (blnContainerControl)
					{
						if (intX >= intLastContainerX && intY >= intLastContainerY)
						{
							intPos = intKtr;
							intLastContainerX = intX;
							intLastContainerY = intY;
						}
					}
					
				}
				
				return intPos;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return -1;
		}
		
		public function isContainerControl(strControlName:String):Boolean
		{
			try{
				switch (strControlName)
				{
					case "com.balsamiq.mockups::TabBar": 
						return true;
					case "com.balsamiq.mockups::BrowserWindow": 
						return true;
					case "com.balsamiq.mockups::Canvas": 
						return true;
					case "com.balsamiq.mockups::FieldSet": 
						return true;
					case "com.balsamiq.mockups::TitleWindow": 
						return true;
						//          case "__group__": 
						//              return true;
				}
				return false;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return false;
		}
		
		public function isChildOfContainer(Parent:Object, Child:Object):Boolean
		{
			try{
				var intParentx:int;
				var intParenty:int;
				var intParentHeight:int;
				var intParentWidth:int;
				var intChildx:int;
				var intChildy:int;
				var intChildHeight:int;
				var intChildWidth:int;
				var blnResult:Boolean = true;
				var strParentID:String = "";
				var strChildGroupID:String = "";
				var blnChildContainerControl:Boolean = false;
				
				blnChildContainerControl = isContainerControl(Child.controlTypeID.toString());
				
				
				
				if (Parent != null)
				{
					intParentx = parseInt(Parent.x);
					intParenty = parseInt(Parent.y);
					intParentWidth = parseInt(Parent.w);
					intParentHeight = parseInt(Parent.h);
					intChildx = parseInt(Child.x);
					intChildy = parseInt(Child.y);1
					intChildWidth = parseInt(Child.w);
					intChildHeight = parseInt(Child.h);
					strChildGroupID = Child.isInGroup.toString();
					strParentID = Parent.controlID.toString();
					
					if (intParentx <= intChildx && intParenty <= intChildy && intParentx + intParentWidth >= intChildx + intChildWidth && intParenty + intParentHeight >= intChildy + intChildHeight)
					{
						blnResult = true;
					}
					else
					{
						// bmml doesn't keep container/contained relationship, it determinates what is contained by physical position (same as logic above)
						// but on real UI, it is very common that some control is slightly out of its container/tab
						// following bmml's rule strictly, it won't be in its container, this will break our UI
						
						if (isContainerControl(Parent.controlTypeID.toString()) )
						{
							
							//							if (/*Parent.controlID == '200339' &&*/ Child.controlID == '8003291') 
							//							{
							//								trace ("parentid=" + Parent.controlID );
							//								trace("parent:x=" + intParentx + ", y=" + intParenty + ", w=" + intParentWidth + ", h=" + intParentHeight);
							//								trace("intParentx + intParentWidth=" +  (intParentx + intParentWidth));
							//								trace("child:x=" + intChildx + ", y=" + intChildy + ", w=" + intChildWidth + ", h=" + intChildHeight);
							//								trace(" 0.9 * (intChildx + intChildWidth)=" + ( 0.9 * (intChildx + intChildWidth)));
							//								trace ("here");
							//							}
							if (intParentx <= intChildx * 0.99 && intParenty <= intChildy * 0.99 && intParentx + intParentWidth >= 0.93 * (intChildx + intChildWidth) && intParenty + intParentHeight >= 0.93 * (intChildy + intChildHeight))
							{
								blnResult = true;
							} else {
								blnResult = false;
							}
						} else {
							
							blnResult = false;
						}
						
					}
					
				}
				
				return blnResult;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return false;
		}
		
		
		private function readParameterInCustomData( full:String, parameter:String):String
		{
			//: paras of main canvas:$GrailsName=Firewall_Route;$RESTUrl=disableREST
			var pattern:RegExp = new RegExp('\\$' + parameter + "=([\\.-_\\w|,]*);", "sg");
			
			var result:Array = pattern.exec(full); 
			if (result != null) {
				return result[1];
			} else {
				var pattern2:RegExp = new RegExp('\\$' + parameter + "=([\\.-_\\w |,]*)$", "sg");
				
				var result2:Array = pattern2.exec(full); 
				if (result2 != null) {
					return result2[1];
				} else {
					return null;		
				}
				
			}
		}
		public function extractDomainClassInformation():void
		{
			// clear old information ( it is possible that someone can use this class to parse more than one files)
			domainClassFields = new Array();
			var processedFields:Array = new Array();
			trace("parse domain class meta information!");
			try{
				arrControls = new Array();
				
				for each (var object:XML in xmlData.controls..*)
				{
					// the Canvas holds information of the Domain class
					if (object.name() == "control" && object.@controlTypeID=="com.balsamiq.mockups::Canvas" &&  object.@controlID=="1")
					{
						var variables:String = object.controlProperties.customData.toString();
						
						trace("paras of main canvas:" + variables);
						GrailsSanitizerType = readParameterInCustomData(variables, "GrailsSanitizerType");
						TableName = readParameterInCustomData(variables, "TableName");
						RESTUrl = readParameterInCustomData(variables, "RESTUrl");
						GrailsDomainClassPackageName = readParameterInCustomData(variables, "GrailsDomainClassPackageName");
						GrailsDomainClassName = readParameterInCustomData(variables, "GrailsDomainClassName");
						
						ColumnFieldsInConfiguration = readParameterInCustomData(variables, "ColumnFields");
						
						
						trace("para:" + readParameterInCustomData(variables, "GrailsSanitizerType"));
						trace("paraGrailsName:" + readParameterInCustomData(variables, "GrailsName"));
						//$GrailsSanitizerType=notSanitizer; $TableName=simple_books; $GrailsDomainClassPackageName=demo; $GrailsDomainClassName=SimpleBook;
						
					}
					
					
					
					// the controls with FieldName holds information of fields of the Domain class
					if (object.name() == "control" && object.controlProperties != null && object.controlProperties.FieldName != null &&object.controlProperties.FieldName.toString().length>0 )
					{
						
						//<customData>$ToGrails=true;$NewDomain=false;$GrailsName=pageCount;$MultivalueFieldType=array;$GeneratedUIFlag=true;$GORMType=Integer;$CreateIndex=false;$DominoType=number;$IsMultivalues=false</customData> 
						
						var ToGrails:String = readParameterInCustomData(object.controlProperties.customData.toString(), "ToGrails");
						if ("true" == ToGrails && processedFields.indexOf(object.controlProperties.FieldName.toString()) < 0 ) {
							processedFields.push(object.controlProperties.FieldName.toString());
							//trace ("test:::::" + object.controlProperties.FieldName.toString());
							
							
							var flexJSComponent:String = componentMapper(object.@controlTypeID);
							
							//CheckBoxListText
							
							var field: Object = new Object();
							field["FlexJSComponent"] = flexJSComponent;
							field["FieldKind"] = object.controlProperties.FieldKind.toString();
							field["ControlType"] = object.controlProperties.ControlType.toString();
							field["customID"] = object.controlProperties.customID.toString();
							field["customData"] = object.controlProperties.customData.toString();
							
							field["GrailsName"] = readParameterInCustomData(object.controlProperties.customData.toString(), "GrailsName");
							field["ToGrails"] = readParameterInCustomData(object.controlProperties.customData.toString(), "ToGrails");
							field["NewDomain"] = readParameterInCustomData(object.controlProperties.customData.toString(), "NewDomain");
							field["OptionList"] = readParameterInCustomData(object.controlProperties.customData.toString(), "OptionList");
							field["CreateIndex"] = readParameterInCustomData(object.controlProperties.customData.toString(), "CreateIndex");
							field["DominoType"] = readParameterInCustomData(object.controlProperties.customData.toString(), "DominoType");
							field["IsMultivalues"] = readParameterInCustomData(object.controlProperties.customData.toString(), "IsMultivalues");
							//trace("[IsMultivalues]=" + field["IsMultivalues"]);
							field["MultivalueFieldType"] = readParameterInCustomData(object.controlProperties.customData.toString(), "MultivalueFieldType");
							field["GeneratedUIFlag"] = readParameterInCustomData(object.controlProperties.customData.toString(), "GeneratedUIFlag");
							
							//<control controlID="800167" controlTypeID="com.balsamiq.mockups::List" x="126" y="181" w="80" h="75" measuredW="80" measuredH="" zOrder="5" locked="false" isInGroup="-1">
							
							field["x"] = object.attribute('x');
							field["y"] =  object.attribute('y');
							field["w"] =  object.attribute('w');
							field["h"] =  object.attribute('h');
							field["measuredW"] = object.attribute('measuredW');
							field["measuredH"] = object.attribute('measuredH');
							field["zOrder"] = object.attribute('zOrder');
							field["locked"] = object.attribute('locked');
							field["isInGroup"] = object.attribute('isInGroup');
							field["controlID"] = object.attribute('controlID');
							field["controlTypeID"] = object.attribute('controlTypeID');
							//measuredW="80" measuredH="" zOrder="5" locked="false" isInGroup="-1"
							
							
							var GORMType:String = readParameterInCustomData(object.controlProperties.customData.toString(), "GORMType");
							
							if (GORMType == null || GORMType == '') 
							{
								GORMType = 'String';
								
							}
							var GORMTypeCore: String = GORMType;
							if (GORMTypeCore != null) 
							{
								GORMTypeCore = GORMTypeCore.replace("]","").replace("[", "");	
							}
							
							
							field["GORMType"] = GORMType;
							field["GORMTypeCore"] = GORMTypeCore;
							
							
							var dominoHideFormulaScript:String = null;
							if (object.controlProperties.dominoHideFormulaScript != null && object.controlProperties.dominoHideFormulaScript.toString() != "") 
							{
								
								dominoHideFormulaScript = object.controlProperties.dominoHideFormulaScript;
							} else {
								dominoHideFormulaScript = "return false;";
								
							}
							field["dominoHideFormulaScript"] = dominoHideFormulaScript;
							domainClassFields.push(field);
						}
						
					}
				}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		public function updateFieldsTabInformation( fieldName: String,  tabName: String):void
		{
			//			trace ("to udpate fields tab:field=" + fieldName + " tab=" + tabName);
			for each (var field:Object in domainClassFields)
			{
				if (fieldName == field["GrailsName"])
					field["Tab"] = tabName; 
			}
		}
		
		public function extractAllControls():void
		{
			try{
				arrControls = new Array();
				
				for each (var object:XML in xmlData.controls..*)
				{
					if (object.name() == "control")
					{
						var strText:String = "";
						var intX:Number = 0;
						var intY:Number = 0;
						var intXY:Number = 0;
						var strSize:String = "";
						var strColor:String = "";
						var strAlign:String = "";
						var strIsInGroup:String = "";
						var strLocked:String = "";
						var strSelectedIndex:String = "";
						
						intX = parseInt(object.@x);
						intY = parseInt(object.@y);
						intXY = intY * 1000 + intX;
						
						strText = object.controlProperties.text.toString();
						strSize = object.controlProperties.size.toString();
						strColor = object.controlProperties.color.toString();
						strAlign = object.controlProperties.align.toString();
						strIsInGroup = object.@isInGroup.toString();
						strLocked = object.@locked.toString();
						strSelectedIndex = object.controlProperties.selectedIndex.toString();
						if (object.controlProperties!= null)
						{
							arrControls.push({controlProperties: object.controlProperties, SelectedIndex: strSelectedIndex, locked: strLocked, isInGroup: strIsInGroup, size: strSize, color: strColor, align: strAlign, text: strText, controlID: object.@controlID.toString(), controlTypeID: object.@controlTypeID.toString(), x: object.@x.toString(), y: object.@y.toString(), w: object.@w.toString(), h: object.@h.toString(), measuredW: object.@measuredW.toString(), measuredH: object.@measuredH.toString(), XY: intXY, IsCustomTAB: false});
						} else {
							arrControls.push({ SelectedIndex: strSelectedIndex,locked: strLocked, isInGroup: strIsInGroup, size: strSize, color: strColor, align: strAlign, text: strText, controlID: object.@controlID.toString(), controlTypeID: object.@controlTypeID.toString(), x: object.@x.toString(), y: object.@y.toString(), w: object.@w.toString(), h: object.@h.toString(), measuredW: object.@measuredW.toString(), measuredH: object.@measuredH.toString(), XY: intXY, IsCustomTAB: false});
						}
					}
				}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		public function extractAllControlsArray():void
		{
			try{
				var intKtr:int = 0;
				var intLen:int = 0;
				var intLastYPos:int = 0;
				var intLastMaxYPos:int = 0;
				var intFirstContainer:int = -1;
				
				arrControls = new Array();
				
				intLen = arrXMLData.length;
				
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					var xmlDataTmp:XML;
					var intOffset:int;
					
					
					xmlDataTmp = arrXMLData[intKtr];
					intOffset = intLastMaxYPos;
					
					
					
					for each (var object:XML in xmlDataTmp.controls..*)
					{
						if (object.name() == "control")
						{
							var strText:String = "";
							var intX:Number = 0;
							var intWidth:Number = 0;
							var intY:Number = 0;
							var intXY:Number = 0;
							var strSize:String = "";
							var strColor:String = "";
							var strAlign:String = "";
							var strIsInGroup:String = "";
							var strLocked:String = "";
							var strSelectedIndex:String = "";
							var strControlTypeID:String = object.@controlTypeID.toString();
							var intH:int = object.@h;
							
							intX = parseInt(object.@x);
							intY = parseInt(object.@y)+intOffset;
							intXY = intY * 1000 + intX;
							intWidth = object.@w;
							
							strText = object.controlProperties.text.toString();
							strSize = object.controlProperties.size.toString();
							strColor = object.controlProperties.color.toString();
							strAlign = object.controlProperties.align.toString();
							strIsInGroup = object.@isInGroup.toString();
							strLocked = object.@locked.toString();
							strSelectedIndex = object.controlProperties.selectedIndex.toString();
							
							intLastYPos = intY + intWidth;
							
							if (intLastYPos > intLastMaxYPos)
								intLastMaxYPos = intLastYPos;
							
							if (strControlTypeID == "com.balsamiq.mockups::Canvas")
							{
								//trace("Canvas");
							}
							
							if (!(strControlTypeID == "com.balsamiq.mockups::Canvas" && intX == 0 && parseInt(object.@y) == 10 && intH == 4096))
							{
								if (object.controlProperties!= null)
								{
									arrControls.push({controlProperties: object.controlProperties, SelectedIndex: strSelectedIndex, locked: strLocked, isInGroup: strIsInGroup, size: strSize, color: strColor, align: strAlign, text: strText, controlID: object.@controlID.toString(), controlTypeID: strControlTypeID, x: intX, y: intY, w: intWidth, h: intH, measuredW: object.@measuredW.toString(), measuredH: object.@measuredH.toString(), XY: intXY, IsCustomTAB: false});
								} else {
									arrControls.push({ SelectedIndex: strSelectedIndex,locked: strLocked, isInGroup: strIsInGroup, size: strSize, color: strColor, align: strAlign, text: strText, controlID: object.@controlID.toString(), controlTypeID: strControlTypeID, x: intX, y: intY, w: intWidth, h: intH, measuredW: object.@measuredW.toString(), measuredH: object.@measuredH.toString(), XY: intXY, IsCustomTAB: false});
								}
							}
							else
							{
								if (intFirstContainer ==-1)
								{
									arrControls.push({controlProperties: object.controlProperties, SelectedIndex: strSelectedIndex, locked: strLocked, isInGroup: strIsInGroup, size: strSize, color: strColor, align: strAlign, text: strText, controlID: object.@controlID.toString(), controlTypeID: strControlTypeID, x: intX, y: intY, w: intWidth, h: intH, measuredW: object.@measuredW.toString(), measuredH: object.@measuredH.toString(), XY: intXY, IsCustomTAB: false});
									intFirstContainer = intKtr;
								}
							}
						}
					}
				}
				if (intFirstContainer !=-1)
				{
					arrControls[intFirstContainer].h = intKtr * 4096;
					arrControls[intFirstContainer].measuredH = intKtr * 4096;
				}
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		
		private var collectedTabNames:Object = new Object();
		
		private function getUniqueTabName(tabName: String) : String
		{
			
			var uniqueTabName:String;
			if (collectedTabNames.hasOwnProperty(tabName)) 
			{
				uniqueTabName = tabName + collectedTabNames[tabName] ;
				collectedTabNames[tabName] += 1;
				
			} else {
				uniqueTabName = tabName;
				collectedTabNames[tabName] = 1;
			}
			return uniqueTabName;
		}
		
		private var _tabGroupId:int = 0;
		
		private function resetTabGroupId() : void
		{
			_tabGroupId = 0;
		}
		private function getNewTabGroupId():String
		{
			// TODO Auto Generated method stub
			_tabGroupId = _tabGroupId + 1;
			return _tabGroupId.toString();
		}
		
		
		private function getCurrentTabGroupId() : String
		{
			return _tabGroupId.toString();
		}
		
		private function insertToTopOfView(view:String, textToInsert:String):String
		{
			var positionToInsert:uint = view.indexOf(">");
			
			return view;
			
		}
		
		
		private function getTABNameByIndex(strText:String, strSelectedIndex:String):String
		{
			var strTABName:String;
			var intSelectedIndex:int;
			
			intSelectedIndex = parseInt(strSelectedIndex);
			
			var arrText:Array;
			
			
			arrText = strText.split(",");
			
			strTABName = arrText[intSelectedIndex];
			
			return strTABName;
			
		}
		
		
		private function applyMargin(intPos:int, intMarginT:int, intMarginR:int, intMarginB:int, intMarginL:int):void
		{
			try{
				var strMarginStyle:String = "";
				var strHTML:String = "";
				var strHTMLNew:String = "";
				var intStylePos:int = -1;           
				
				strHTML = arrComponents[intPos].text;
				intStylePos = strHTML.indexOf("style=");
				
				if (intStylePos == -1)
				{
					strMarginStyle = "margin:" + intMarginT.toString() + "px " + intMarginR.toString() + "px " + intMarginB.toString() + "px " + intMarginL.toString() + "px"
					intStylePos = strHTML.indexOf(">");
					strHTMLNew = strHTML.slice(0, intStylePos) + " style=\"" + strMarginStyle + "\"" + strHTML.slice(intStylePos);
				}
				else
				{
					strMarginStyle = "margin:" + intMarginT.toString() + "px " + intMarginR.toString() + "px " + intMarginB.toString() + "px " + intMarginL.toString() + "px;"
					strHTMLNew = strHTML.slice(0, intStylePos+7) + strMarginStyle + strHTML.slice(intStylePos+7);
				}
				
				arrComponents[intPos].text = strHTMLNew;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
		}
		
		private function trimArray(arrToTrim:Array):Array
		{
			try{
				var intLen:int = 0;
				var intKtr:int = 0;
				
				var arrTrimmed:Array = new Array();
				
				intLen = arrToTrim.length;
				
				for (intKtr = 0; intKtr < intLen; intKtr++)
				{
					if (arrToTrim[intKtr] != "")
					{
						arrTrimmed.push(arrToTrim[intKtr]);
					}
				}
				
				return arrTrimmed;
			}
			catch (e:Error)
			{
				trace(e.getStackTrace()); Alert.show(e.getStackTrace());
			}
			return null;
		}
		
		private function propertyMapper(strName:String):String
		{
			return unescape(strName);
		}
		
		
		
		private function convertTextOrFormula( rawText: String) : String
		{
			
			var pattern:RegExp = /@.*\(.*\)/;
			if (pattern.exec(rawText) )
			{
				
				return rawText.replace("@", "DominoFunction_");
				
			} else {
				return rawText;
			}
		}
		
		
		
		
		
		
		
		
		// Functions to manage templates of this converter
		private const DOMAINCLASSUIUTILSTEMPLATE:String = "DomainClassUiUtils.as";
		private function getDomainClassUiUtilsFileName() : String
		{
			return strMXMLExportFilePath + "/src/" + getPackageAsFilePath()+"/" +getDomainName() + "UiUtils.as";
		}
		private function exportUiUtils():void
		{
			// TODO 
			// dummy file for now
			// it is possible that custom template system isn't enough
			var targetFileName:String = getDomainClassUiUtilsFileName();
			copyTemplateFileTo(DOMAINCLASSUIUTILSTEMPLATE, targetFileName);
		}
		
		
		
		
		private const DOMAINCLASSTABLEITEMRENDERERTEMPLATE:String = "DomainClassTableRowItemRenderer.mxml";
		private function getTableItemRenderFileName() : String
		{
			return strMXMLExportFilePath + "/src/views/" +getDomainName() +"/" + getDomainName() + "TableRowItemRenderer.mxml";
		}
		private function exportTableItemRenderer():void
		{
			// TODO 
			// dummy file for now
			// it is possible that custom template system isn't enough
			var targetFileName:String = getTableItemRenderFileName();
			trace(new File(targetFileName).exists);
			copyTemplateFileTo(DOMAINCLASSTABLEITEMRENDERERTEMPLATE, targetFileName);
		}
		
		
		private const DOMAINCLASSTEMPLATE:String = "DomainClass.as";
		private function getDomainClassFileName() : String
		{
			return strMXMLExportFilePath + "/src/" + getPackageAsFilePath()+"/" +getDomainName() + ".as";
		}
		private function exportDomainClass():void
		{
			// TODO 
			// dummy file for now
			// it is possible that custom template system isn't enough
			var targetFileName:String = getDomainClassFileName();
			trace(new File(targetFileName).exists);
			copyTemplateFileTo(DOMAINCLASSTEMPLATE, targetFileName);
		}
		
		
		
		private const DOMAINCLASSUTILSTEMPLATE:String = "DomainClassUtils.as";
		private function getDomainClassUtilsFileName() : String
		{
			return strMXMLExportFilePath + "/src/" + getPackageAsFilePath()+"/"  +getDomainName() + "Utils.as";
		}
		private function exportDomainClassUtils():void
		{
			// TODO 
			// dummy file for now
			// it is possible that custom template system isn't enough
			var targetFileName:String = getDomainClassUtilsFileName();
			copyTemplateFileTo(DOMAINCLASSUTILSTEMPLATE, targetFileName);
		}
		
		
		// moonshineide project file (fixed file content with variables)
		private function getProjectFileName( single:Boolean ):String
		{
			if (single)
			    return strMXMLExportFilePath + "/" +getDomainName() + ".as3proj";
			else 
				return strMXMLExportFilePath + "/FindABetterNameMayBeAInputParameter.as3proj";
		}
		private const PROJECTTEMPLATE:String = "project.as3proj";
		private function exportProjectFile( single:Boolean ):void
		{
			var projectFileName:String = getProjectFileName(single);
			// it is possible that this step is already done by moonshineide
			copyTemplateFileTo(PROJECTTEMPLATE, projectFileName);
		}
		
		// object list view (fixed file content with variables)
		//private const LISTVIEWTEMPLATE:String = "templates/domainList.mxml";
		private const LISTVIEWTEMPLATE:String = "domainList.mxml";
		
		private function exportObjectListView():void
		{
			
			var listViewName:String = getListViewFileName();
			copyTemplateFileTo(LISTVIEWTEMPLATE, listViewName);
			
		}
		
		private const MAINVIEWTEMPLATE:String = "MainView.mxml";
		private const MAINVIEWSINGLETEMPLATE:String = "MainViewOfSingleDomainProject.mxml";
		private const NAVIGATORVIEWTEMPLATE:String = "NavigatorView.mxml";
		
		private function exportMainViewsOfSingleDomainProject( single:Boolean):void
		{
			var mainViewName:String = getMainViewOfSingleDomainProjectFileName();
			copyTemplateFileTo(MAINVIEWSINGLETEMPLATE, mainViewName);
		}
		
		
		private function exportMainViews( single:Boolean):void
		{
			var mainViewName:String = getMainViewFileName();
			copyTemplateFileTo(MAINVIEWTEMPLATE, mainViewName);
			var navigatorViewName:String = getNavigatorViewFileName();
			copyTemplateFileTo(NAVIGATORVIEWTEMPLATE, navigatorViewName);
		}
		
		private function getMainViewOfSingleDomainProjectFileName():String
		{
			// some to project name, with .mxml
			//return strMXMLExportFilePath + "/src/"  +getDomainName() + "List.mxml";
			return strMXMLExportFilePath + "/src/" + "MainView.mxml";
		}
		
		private function getMainViewFileName():String
		{
			// some to project name, with .mxml
			//return strMXMLExportFilePath + "/src/"  +getDomainName() + "List.mxml";
			return strMXMLExportFilePath + "/src/" + "MainView.mxml";
		}
		
		private function getNavigatorViewFileName():String
		{
			// some to project name, with .mxml
			//return strMXMLExportFilePath + "/src/"  +getDomainName() + "List.mxml";
			return strMXMLExportFilePath + "/src/" + "NavigatorView.mxml";
		}
		
		private function getListViewFileName():String
		{
			// some to project name, with .mxml
			return strMXMLExportFilePath + "/src/views/"  + getDomainName() +  "ListView.mxml";
			//return strMXMLExportFilePath + "/src/views/MyInitialView.mxml";
		}
		
		
		
		private function getDataAccessLayerFileName():String
		{
			return strMXMLExportFilePath + "/src/" + getPackageAsFilePath()+"/"  +getDomainName() + "Manager.as";
			
		}
		// data access layer (fixed file content with variables)
		private const DATAACCESSLAYERNAME:String = "DomainClassManager.as";
		private function exportDataAccessLayer():void
		{
			var dataAccessLayerName:String = getDataAccessLayerFileName();
			copyTemplateFileTo(DATAACCESSLAYERNAME, dataAccessLayerName);
		}
		
		
		
		// Functions process templates
		private function copyStaticFileTo(templateFileName:String, relativeTargetFileName: String): void
		{
			copyTemplateFileTo(templateFileName, strMXMLExportFilePath + "/" + relativeTargetFileName);
		}
		private function copyTemplateFileTo( templateFileName:String,  targetFileName:String): void
		{
			
			var myXMLURL:URLRequest = new URLRequest((IS_MACOS ? "file://" : "")+ templatesFolder.resolvePath(templateFileName).nativePath);
			var myLoader:URLLoader = new URLLoader();
			
			myLoader.addEventListener(Event.COMPLETE,  function(e:Event) : void { 
				CopyTemplateFileTO_templateReaded(e.target.data, targetFileName) 
			} );
			
			myLoader.load(myXMLURL);
		}   
		
		private function CopyTemplateFileTO_templateReaded(templateContent:String, targetFileName:String) : void
		{
			
			var parameters: Object = new Object();
			
			parameters["TableName"] = TableName;
			parameters["RESTUrl"] = RESTUrl;
			
			parameters["GrailsDomainClassPackageName"] = GrailsDomainClassPackageName;
			var ColumnFields : Array;
			if (ColumnFieldsInConfiguration != null && ColumnFieldsInConfiguration.length > 0)
			{
				ColumnFields = ColumnFieldsInConfiguration.split(",");
				if (ColumnFields[ColumnFields.length - 1] == '' || ColumnFields[ColumnFields.length - 1] == null) ColumnFields.removeAt(ColumnFields.length - 1);
			} else {
				ColumnFields = domainClassFields;
				ColumnFieldsInConfiguration = "";
			}
			
			
			var ColumnFields2 : Array = new Array();
			
			if (ColumnFieldsInConfiguration != null && ColumnFieldsInConfiguration.length > 0)
			{
				ColumnFields = ColumnFieldsInConfiguration.split(",");
				//if (ColumnFields[ColumnFields.length - 1] == '' || ColumnFields[ColumnFields.length - 1] == null) ColumnFields.removeAt(ColumnFields.length - 1);
				for each (var field:Object in domainClassFields)
				{
					if (ColumnFieldsInConfiguration.indexOf(field["GrailsName"] + ",") > -1 || ColumnFieldsInConfiguration.indexOf(field["GrailsName"] + ";") > -1)
					{
						ColumnFields2.push(field);
					}
					
				}
			} else {
				ColumnFields2 = domainClassFields;
				ColumnFieldsInConfiguration = "";
			}
			
			
			parameters["ColumnFields"] = ColumnFields2;
			parameters["GrailsDomainClassName"] = GrailsDomainClassName;
			parameters["GrailsSanitizerType"] = GrailsSanitizerType;
			
			parameters["domainClassFields"] = domainClassFields;
			
			
			var templateManager : TemplateManager = new TemplateManager();
			templateManager.CopyTemplateFileTO_templateReaded(templateContent, targetFileName, parameters);
		}
		
	}
}
